import { jsx, jsxs } from "react/jsx-runtime";
import React__default from "react";
import { x2c, y2r } from "../lib/converters.js";
import { zoneToArea, among, areaToRange, areaToZone, zoneShape } from "../lib/structs.js";
import { setEditorRect, setInputting, setAutofillDraggingTo, drag, updateTable, select, choose, setEditingCell, setContextMenuPosition, write } from "../store/actions.js";
import { DUMMY_IMG } from "../constants.js";
import { Context } from "../store/index.js";
import { FormulaError } from "../formula/evaluator.js";
import { Autofill } from "../lib/autofill.js";
import { insertRef, isRefInsertable } from "../lib/input.js";
import { useSheetContext } from "./SheetProvider.js";
const Cell = React__default.memo(({ y, x, operationStyle }) => {
  const rowId = y2r(y);
  const colId = x2c(x);
  const address = `${colId}${rowId}`;
  const { store, dispatch } = React__default.useContext(Context);
  const isFirstPointed = React__default.useRef(true);
  const [sheetProvided, sheetContext] = useSheetContext();
  const cellRef = React__default.useRef(null);
  const {
    table,
    editingCell,
    choosing,
    selectingZone,
    leftHeaderSelecting,
    topHeaderSelecting,
    editorRef,
    showAddress,
    autofillDraggingTo,
    lastEdited
  } = store;
  const differentSheetFocused = sheetProvided && (sheetContext == null ? void 0 : sheetContext.lastFocusedRef) !== store.lastFocusedRef;
  const lastFocusedRef = (sheetContext == null ? void 0 : sheetContext.lastFocusedRef) || store.lastFocusedRef;
  const selectingArea = zoneToArea(selectingZone);
  const editing = editingCell === address;
  const xEditing = !sheetProvided || (sheetContext == null ? void 0 : sheetContext.editingCell) === (sheetContext == null ? void 0 : sheetContext.choosingCell);
  const pointed = choosing.y === y && choosing.x === x;
  const _setEditorRect = React__default.useCallback(() => {
    const rect = cellRef.current.getBoundingClientRect();
    dispatch(
      setEditorRect({
        y: rect.y,
        x: rect.x,
        height: rect.height,
        width: rect.width
      })
    );
  }, []);
  React__default.useEffect(() => {
    if (pointed && !isFirstPointed.current) {
      _setEditorRect();
      if (!editing) {
        dispatch(setInputting(table.stringify({ y, x })));
      }
      return;
    }
    isFirstPointed.current = false;
  }, [pointed, editing]);
  const cell = table.getByPoint({ y, x });
  const writeCell = (value) => {
    if (lastEdited !== value) {
      dispatch(write(value));
      return;
    }
  };
  let errorMessage = "";
  let rendered;
  try {
    rendered = table.render({ y, x }, writeCell);
  } catch (e) {
    if (e instanceof FormulaError) {
      errorMessage = e.message;
      rendered = e.code;
    } else {
      errorMessage = e.message;
      rendered = "#UNKNOWN";
      console.error(e);
    }
  }
  const lastInput = lastFocusedRef.current;
  const input = editorRef.current;
  if (!input) {
    return null;
  }
  return /* @__PURE__ */ jsx(
    "td",
    {
      ref: cellRef,
      "data-x": x,
      "data-y": y,
      "data-address": address,
      className: `gs-cell ${among(selectingArea, { y, x }) ? "gs-selecting" : ""} ${pointed ? "gs-choosing" : ""} ${editing ? "gs-editing" : ""}`,
      style: {
        ...cell == null ? void 0 : cell.style,
        ...operationStyle
      },
      onContextMenu: (e) => {
        e.preventDefault();
        dispatch(setContextMenuPosition({ y: e.clientY, x: e.clientX }));
        return false;
      },
      onClick: (e) => {
        if (autofillDraggingTo) {
          return false;
        }
        const fullAddress = `${table.sheetPrefix(!differentSheetFocused)}${address}`;
        const editing2 = !!((sheetContext == null ? void 0 : sheetContext.editingCell) || editingCell);
        if (editing2) {
          const inserted = insertRef(lastInput, fullAddress);
          if (inserted) {
            return false;
          }
        }
        dispatch(setEditingCell(""));
        dispatch(setContextMenuPosition({ y: -1, x: -1 }));
        input.focus();
        if (e.shiftKey) {
          dispatch(drag({ y, x }));
          return;
        } else {
          dispatch(choose({ y, x }));
          dispatch(select({ startY: y, startX: x, endY: -1, endX: -1 }));
          _setEditorRect();
        }
        const valueString = table.stringify({ y, x });
        dispatch(setInputting(valueString));
      },
      onDoubleClick: (e) => {
        e.preventDefault();
        setEditingCell(address);
        const dblclick = document.createEvent("MouseEvents");
        dblclick.initEvent("dblclick", true, true);
        input.dispatchEvent(dblclick);
        return false;
      },
      draggable: true,
      onDragStart: (e) => {
        if (autofillDraggingTo) {
          return false;
        }
        e.dataTransfer.setDragImage(DUMMY_IMG, 0, 0);
        dispatch(select({ startY: y, startX: x, endY: y, endX: x }));
        const insertable = isRefInsertable(lastInput);
        if (insertable && xEditing) {
          return true;
        } else if (insertable != null) {
          writeCell(input.value);
        }
        dispatch(choose({ y, x }));
        input.focus();
        dispatch(setInputting(""));
      },
      onDragEnd: () => {
        var _a;
        if (autofillDraggingTo) {
          if (autofillDraggingTo.x !== x || autofillDraggingTo.y !== y) {
            const autofill = new Autofill(store, autofillDraggingTo);
            dispatch(updateTable(autofill.applied));
            dispatch(select(areaToZone(autofill.wholeArea)));
            input.focus();
          }
          dispatch(setAutofillDraggingTo(null));
          return false;
        }
        const { height: h, width: w } = zoneShape(selectingZone);
        if (h + w === 0) {
          dispatch(select({ startY: -1, startX: -1, endY: -1, endX: -1 }));
        }
        if (isRefInsertable(lastInput)) {
          dispatch(select({ startY: -1, startX: -1, endY: -1, endX: -1 }));
        }
        (_a = lastFocusedRef.current) == null ? void 0 : _a.focus();
      },
      onDragEnter: () => {
        var _a;
        if (autofillDraggingTo) {
          if (!among(selectingArea, { x, y })) {
            dispatch(setAutofillDraggingTo({ x, y }));
          }
          return false;
        }
        if (leftHeaderSelecting) {
          dispatch(drag({ y: table.getNumRows(), x }));
          return false;
        }
        if (topHeaderSelecting) {
          dispatch(drag({ y, x: table.getNumCols() }));
          return false;
        }
        dispatch(drag({ y, x }));
        const newArea = zoneToArea({ ...selectingZone, endY: y, endX: x });
        const fullRange = `${table.sheetPrefix(!differentSheetFocused)}${areaToRange(newArea)}`;
        insertRef(lastInput, fullRange);
        (_a = sheetContext == null ? void 0 : sheetContext.forceRender) == null ? void 0 : _a.call(sheetContext);
        return true;
      },
      children: /* @__PURE__ */ jsxs("div", { className: `gs-cell-inner-wrap`, children: [
        /* @__PURE__ */ jsxs(
          "div",
          {
            className: "gs-cell-inner",
            style: {
              ...cell == null ? void 0 : cell.style,
              justifyContent: (cell == null ? void 0 : cell.justifyContent) || "left",
              alignItems: (cell == null ? void 0 : cell.alignItems) || "start"
            },
            children: [
              errorMessage && /* @__PURE__ */ jsx("div", { className: "gs-formula-error-triangle", title: errorMessage }),
              showAddress && /* @__PURE__ */ jsx("div", { className: "gs-cell-label", children: address }),
              /* @__PURE__ */ jsx("div", { className: "gs-cell-rendered", children: rendered })
            ]
          }
        ),
        (!editing && pointed && selectingArea.bottom === -1 || selectingArea.bottom === y && selectingArea.right === x) && /* @__PURE__ */ jsx(
          "div",
          {
            className: "gs-autofill-drag",
            draggable: true,
            onDragStart: (e) => {
              e.dataTransfer.setDragImage(DUMMY_IMG, 0, 0);
              dispatch(setAutofillDraggingTo({ x, y }));
              e.stopPropagation();
            }
          }
        )
      ] })
    },
    x
  );
});
export {
  Cell
};
//# sourceMappingURL=Cell.js.map
