import { jsx, jsxs } from "react/jsx-runtime";
import React__default from "react";
import { clip } from "../lib/clipboard.js";
import { cut, copy, paste, updateTable, undo, redo, setContextMenuPosition } from "../store/actions.js";
import { zoneToArea, zoneShape, areaToZone } from "../lib/structs.js";
import { Context } from "../store/index.js";
import { isPrevented, AddRowAbove, AddRowBelow, AddColLeft, AddColRight, DeleteRow, DeleteCol } from "../lib/prevention.js";
import { Fixed } from "./Fixed.js";
const ContextMenu = () => {
  const { store, dispatch } = React__default.useContext(Context);
  const { table, choosing, selectingZone, leftHeaderSelecting, topHeaderSelecting, editorRef, contextMenuPosition } = store;
  const { y, x } = choosing;
  let {
    top: selectingTop,
    left: selectingLeft,
    bottom: selectingBottom,
    right: selectingRight
  } = zoneToArea(selectingZone);
  if (selectingTop === -1) {
    [selectingTop, selectingLeft, selectingBottom, selectingRight] = [y, x, y, x];
  }
  const [tableHeight, tableWidth] = [table.getNumRows(), table.getNumCols()];
  const { height, width } = zoneShape({ ...selectingZone, base: 1 });
  const { y: top, x: left } = contextMenuPosition;
  if (top === -1) {
    return null;
  }
  const selectingTopCell = table.getByPoint({ y: selectingTop, x: 0 });
  const selectingLeftCell = table.getByPoint({ y: 0, x: selectingLeft });
  const selectingBottomCell = table.getByPoint({ y: selectingBottom, x: 0 });
  const selectingRightCell = table.getByPoint({ y: 0, x: selectingRight });
  const historyIndex = table.getHistoryIndex();
  return /* @__PURE__ */ jsx(
    Fixed,
    {
      className: "gs-contextmenu-modal",
      onClick: (e) => {
        e.preventDefault();
        dispatch(setContextMenuPosition({ y: -1, x: -1 }));
        return false;
      },
      children: /* @__PURE__ */ jsx(
        "div",
        {
          className: "gs-contextmenu",
          style: {
            top,
            left
          },
          children: /* @__PURE__ */ jsxs("ul", { children: [
            /* @__PURE__ */ jsxs(
              "li",
              {
                className: "gs-enabled",
                onClick: () => {
                  const area = clip(store);
                  dispatch(cut(areaToZone(area)));
                },
                children: [
                  /* @__PURE__ */ jsx("div", { className: "gs-menu-name", children: "Cut" }),
                  /* @__PURE__ */ jsx("div", { className: "gs-menu-shortcut", children: /* @__PURE__ */ jsx("span", { className: "gs-menu-underline", children: "X" }) })
                ]
              }
            ),
            /* @__PURE__ */ jsxs(
              "li",
              {
                className: "gs-enabled",
                onClick: () => {
                  const area = clip(store);
                  dispatch(copy(areaToZone(area)));
                },
                children: [
                  /* @__PURE__ */ jsx("div", { className: "gs-menu-name", children: "Copy" }),
                  /* @__PURE__ */ jsx("div", { className: "gs-menu-shortcut", children: /* @__PURE__ */ jsx("span", { className: "gs-menu-underline", children: "C" }) })
                ]
              }
            ),
            /* @__PURE__ */ jsxs(
              "li",
              {
                className: "gs-enabled",
                onClick: () => {
                  var _a;
                  const text = ((_a = editorRef.current) == null ? void 0 : _a.value) || "";
                  dispatch(paste({ text }));
                },
                children: [
                  /* @__PURE__ */ jsx("div", { className: "gs-menu-name", children: "Paste" }),
                  /* @__PURE__ */ jsx("div", { className: "gs-menu-shortcut", children: /* @__PURE__ */ jsx("span", { className: "gs-menu-underline", children: "V" }) })
                ]
              }
            ),
            (leftHeaderSelecting || topHeaderSelecting) && /* @__PURE__ */ jsx("li", { className: "gs-menu-divider" }),
            leftHeaderSelecting && /* @__PURE__ */ jsx(
              "li",
              {
                className: table.maxNumRows !== -1 && tableHeight + height > table.maxNumRows || isPrevented(selectingTopCell == null ? void 0 : selectingTopCell.prevention, AddRowAbove) ? "gs-disabled" : "gs-enabled",
                onClick: () => {
                  const newTable = table.addRows({
                    y: selectingTop,
                    numRows: height,
                    baseY: selectingTop,
                    reflection: {
                      selectingZone,
                      choosing
                    }
                  });
                  dispatch(updateTable(newTable));
                },
                children: /* @__PURE__ */ jsxs("div", { className: "gs-menu-name", children: [
                  "Insert ",
                  height,
                  " row",
                  height > 1 && "s",
                  " above"
                ] })
              }
            ),
            leftHeaderSelecting && /* @__PURE__ */ jsx(
              "li",
              {
                className: table.maxNumRows !== -1 && tableHeight + height > table.maxNumRows || isPrevented(selectingBottomCell == null ? void 0 : selectingBottomCell.prevention, AddRowBelow) ? "gs-disabled" : "gs-enabled",
                onClick: (e) => {
                  if (e.currentTarget.classList.contains("gs-disabled")) {
                    return;
                  }
                  selectingZone.startY += height;
                  selectingZone.endY += height;
                  choosing.y += height;
                  const newTable = table.addRows({
                    y: selectingBottom + 1,
                    numRows: height,
                    baseY: selectingBottom,
                    reflection: {
                      selectingZone,
                      choosing
                    }
                  });
                  dispatch(updateTable(newTable));
                },
                children: /* @__PURE__ */ jsxs("div", { className: "gs-menu-name", children: [
                  "Insert ",
                  height,
                  " row",
                  height > 1 && "s",
                  " below"
                ] })
              }
            ),
            topHeaderSelecting && /* @__PURE__ */ jsx(
              "li",
              {
                className: table.maxNumCols !== -1 && tableWidth + width > table.maxNumCols || isPrevented(selectingLeftCell == null ? void 0 : selectingLeftCell.prevention, AddColLeft) ? "gs-disabled" : "gs-enabled",
                onClick: (e) => {
                  if (e.currentTarget.classList.contains("gs-disabled")) {
                    return;
                  }
                  const newTable = table.addCols({
                    x: selectingLeft,
                    numCols: width,
                    baseX: selectingLeft,
                    reflection: {
                      selectingZone,
                      choosing
                    }
                  });
                  dispatch(updateTable(newTable));
                },
                children: /* @__PURE__ */ jsxs("div", { className: "gs-menu-name", children: [
                  "Insert ",
                  width,
                  " column",
                  width > 1 && "s",
                  " left"
                ] })
              }
            ),
            topHeaderSelecting && /* @__PURE__ */ jsx(
              "li",
              {
                className: table.maxNumCols !== -1 && tableWidth + width > table.maxNumCols || isPrevented(selectingRightCell == null ? void 0 : selectingRightCell.prevention, AddColRight) ? "gs-disabled" : "gs-enabled",
                onClick: (e) => {
                  if (e.currentTarget.classList.contains("gs-disabled")) {
                    return;
                  }
                  selectingZone.startX += width;
                  selectingZone.endX += width;
                  choosing.x += width;
                  const newTable = table.addCols({
                    x: selectingRight + 1,
                    numCols: width,
                    baseX: selectingRight,
                    reflection: {
                      selectingZone,
                      choosing
                    }
                  });
                  dispatch(updateTable(newTable));
                },
                children: /* @__PURE__ */ jsxs("div", { className: "gs-menu-name", children: [
                  "Insert ",
                  width,
                  " column",
                  width > 1 && "s",
                  " right"
                ] })
              }
            ),
            leftHeaderSelecting && /* @__PURE__ */ jsx(
              "li",
              {
                className: table.minNumRows !== -1 && tableHeight - height < table.minNumRows || isPrevented(selectingTopCell == null ? void 0 : selectingTopCell.prevention, DeleteRow) ? "gs-disabled" : "gs-enabled",
                onClick: (e) => {
                  if (e.currentTarget.classList.contains("gs-disabled")) {
                    return;
                  }
                  const newTable = table.deleteRows({
                    y: selectingTop,
                    numRows: height,
                    operator: "USER",
                    reflection: {
                      selectingZone,
                      choosing
                    }
                  });
                  dispatch(updateTable(newTable));
                },
                children: /* @__PURE__ */ jsxs("div", { className: "gs-menu-name", children: [
                  "Delete ",
                  height,
                  " row",
                  height > 1 && "s"
                ] })
              }
            ),
            topHeaderSelecting && /* @__PURE__ */ jsx(
              "li",
              {
                className: table.minNumCols !== -1 && tableWidth - width < table.minNumCols || isPrevented(selectingRightCell == null ? void 0 : selectingRightCell.prevention, DeleteCol) ? "gs-disabled" : "gs-enabled",
                onClick: () => {
                  const newTable = table.deleteCols({
                    x: selectingLeft,
                    numCols: width,
                    operator: "USER",
                    reflection: {
                      selectingZone,
                      choosing
                    }
                  });
                  dispatch(updateTable(newTable));
                },
                children: /* @__PURE__ */ jsxs("div", { className: "gs-menu-name", children: [
                  "Delete ",
                  width,
                  " column",
                  width > 0 && "s"
                ] })
              }
            ),
            (historyIndex > -1 || historyIndex < table.getHistorySize() - 1) && /* @__PURE__ */ jsx("li", { className: "gs-menu-divider" }),
            historyIndex > -1 && /* @__PURE__ */ jsxs(
              "li",
              {
                className: "gs-enabled",
                onClick: () => {
                  dispatch(undo(null));
                },
                children: [
                  /* @__PURE__ */ jsx("div", { className: "gs-menu-name", children: "Undo" }),
                  /* @__PURE__ */ jsx("div", { className: "gs-menu-shortcut", children: /* @__PURE__ */ jsx("span", { className: "gs-menu-underline", children: "Z" }) })
                ]
              }
            ),
            historyIndex < table.getHistorySize() - 1 && /* @__PURE__ */ jsxs(
              "li",
              {
                className: "gs-enabled",
                onClick: () => {
                  dispatch(redo(null));
                },
                children: [
                  /* @__PURE__ */ jsx("div", { className: "gs-menu-name", children: "Redo" }),
                  /* @__PURE__ */ jsx("div", { className: "gs-menu-shortcut", children: /* @__PURE__ */ jsx("span", { className: "gs-menu-underline", children: "R" }) })
                ]
              }
            )
          ] })
        }
      )
    }
  );
};
export {
  ContextMenu
};
//# sourceMappingURL=ContextMenu.js.map
