import { jsxs, jsx, Fragment } from "react/jsx-runtime";
import React__default from "react";
import { x2c, y2r } from "../lib/converters.js";
import { clip } from "../lib/clipboard.js";
import { setEditingCell, setInputting, setLastEdited, setLastFocusedRef, redo, undo, cut, paste, setSearchQuery, setEntering, copy, select, arrow, escape, clear, walk, write } from "../store/actions.js";
import { Context } from "../store/index.js";
import { zoneToArea, areaToZone } from "../lib/structs.js";
import { isPrevented, Write } from "../lib/prevention.js";
import { expandInput, insertTextAtCursor } from "../lib/input.js";
import { useSheetContext } from "./SheetProvider.js";
import { Lexer } from "../formula/evaluator.js";
import { REF_PALETTE } from "../lib/palette.js";
import { Fixed } from "./Fixed.js";
const Editor = ({ mode, handleKeyUp }) => {
  var _a, _b;
  const { store, dispatch } = React__default.useContext(Context);
  const {
    showAddress,
    editorRect,
    editingCell,
    choosing,
    inputting,
    selectingZone,
    searchQuery,
    editorRef,
    largeEditorRef,
    searchInputRef,
    editingOnEnter,
    onSave,
    table,
    sheetId
  } = store;
  const [, sheetContext] = useSheetContext();
  React__default.useEffect(() => {
    var _a2, _b2;
    (_b2 = (_a2 = editorRef == null ? void 0 : editorRef.current) == null ? void 0 : _a2.focus) == null ? void 0 : _b2.call(_a2, { preventScroll: true });
  }, [editorRef]);
  React__default.useEffect(() => {
    if (!(sheetContext == null ? void 0 : sheetContext.lastFocusedRef)) {
      return;
    }
    if (sheetContext.lastFocusedRef === editorRef) {
      return;
    }
    if (sheetContext.lastFocusedRef === largeEditorRef) {
      return;
    }
    dispatch(setEditingCell(""));
  }, [sheetContext == null ? void 0 : sheetContext.lastFocusedRef]);
  React__default.useEffect(() => {
    var _a2;
    (_a2 = sheetContext == null ? void 0 : sheetContext.setEditingCell) == null ? void 0 : _a2.call(sheetContext, editingCell);
  }, [editingCell]);
  const { y, x } = choosing;
  const rowId = `${y2r(y)}`;
  const colId = x2c(x);
  const address = `${colId}${rowId}`;
  const editing = editingCell === address;
  const cell = table.getByPoint({ y, x });
  const value = cell == null ? void 0 : cell.value;
  const valueString = table.stringify({ y, x }, value);
  const [before, setBefore] = React__default.useState(valueString);
  React__default.useEffect(() => {
    setBefore(valueString);
  }, [choosing]);
  const { y: top, x: left, height, width } = editorRect;
  const writeCell = (value2) => {
    if (before !== value2) {
      dispatch(write(value2));
    }
    setBefore(value2);
  };
  const numLines = valueString.split("\n").length;
  const [isKeyDown, setIsKeyDown] = React__default.useState(false);
  const handleKeyDown = (e) => {
    if (isKeyDown) {
      return;
    }
    if (!(e.key === "Meta" || e.key === "Control")) {
      setIsKeyDown(true);
      const timeout = window.setTimeout(() => {
        setIsKeyDown(false);
        window.clearTimeout(timeout);
      }, 10);
    }
    const input = e.currentTarget;
    const shiftKey = e.shiftKey;
    switch (e.key) {
      case "Tab":
        e.preventDefault();
        if (editing) {
          writeCell(input.value);
          dispatch(setEditingCell(""));
          dispatch(setInputting(""));
        }
        dispatch(
          walk({
            numRows: table.getNumRows(),
            numCols: table.getNumCols(),
            deltaY: 0,
            deltaX: shiftKey ? -1 : 1
          })
        );
        dispatch(setEditingCell(""));
        resetSize(e.currentTarget);
        return false;
      case "Enter":
        if (editing) {
          if (e.altKey) {
            insertTextAtCursor(input, "\n");
            dispatch(setInputting(input.value));
            e.preventDefault();
            return false;
          } else {
            if (e.nativeEvent.isComposing) {
              return false;
            }
            writeCell(input.value);
            dispatch(setEditingCell(""));
          }
          resetSize(e.currentTarget);
        } else if (editingOnEnter && selectingZone.endY === -1) {
          const dblclick = document.createEvent("MouseEvents");
          dblclick.initEvent("dblclick", true, true);
          input.dispatchEvent(dblclick);
          e.preventDefault();
          return false;
        }
        dispatch(
          walk({
            numRows: table.getNumRows(),
            numCols: table.getNumCols(),
            deltaY: shiftKey ? -1 : 1,
            deltaX: 0
          })
        );
        e.preventDefault();
        return false;
      case "Backspace":
        if (!editing) {
          dispatch(clear(null));
          return false;
        }
        break;
      case "Shift":
        return false;
      case "Control":
        return false;
      case "Alt":
        return false;
      case "Meta":
        return false;
      case "NumLock":
        return false;
      case "Escape":
        dispatch(escape(null));
        dispatch(setSearchQuery(void 0));
        return false;
      case "ArrowLeft":
        if (!editing) {
          dispatch(
            arrow({
              shiftKey,
              numRows: table.getNumRows(),
              numCols: table.getNumCols(),
              deltaY: 0,
              deltaX: -1
            })
          );
          return false;
        }
        break;
      case "ArrowUp":
        if (!editing) {
          dispatch(
            arrow({
              shiftKey,
              numRows: table.getNumRows(),
              numCols: table.getNumCols(),
              deltaY: -1,
              deltaX: 0
            })
          );
          return false;
        }
        break;
      case "ArrowRight":
        if (!editing) {
          dispatch(
            arrow({
              shiftKey,
              numRows: table.getNumRows(),
              numCols: table.getNumCols(),
              deltaY: 0,
              deltaX: 1
            })
          );
          return false;
        }
        break;
      case "ArrowDown":
        if (!editing) {
          dispatch(
            arrow({
              shiftKey,
              numRows: table.getNumRows(),
              numCols: table.getNumCols(),
              deltaY: 1,
              deltaX: 0
            })
          );
          return false;
        }
        break;
      case "a":
        if (e.ctrlKey || e.metaKey) {
          if (!editing) {
            e.preventDefault();
            dispatch(
              select({
                startY: 1,
                startX: 1,
                endY: table.getNumRows(),
                endX: table.getNumCols()
              })
            );
            return false;
          }
        }
        break;
      case "c":
        if (e.ctrlKey || e.metaKey) {
          if (!editing) {
            e.preventDefault();
            const area = clip(store);
            dispatch(copy(areaToZone(area)));
            input.focus();
            return false;
          }
        }
        break;
      case "f":
        if (e.ctrlKey || e.metaKey) {
          if (!editing) {
            e.preventDefault();
            if (typeof searchQuery === "undefined") {
              dispatch(setSearchQuery(""));
            }
            dispatch(setEntering(false));
            window.setTimeout(() => searchInputRef.current.focus(), 100);
            return false;
          }
        }
        break;
      case "r":
        if (e.ctrlKey || e.metaKey) {
          if (!editing) {
            e.preventDefault();
            dispatch(redo(null));
            window.setTimeout(() => dispatch(setInputting("")), 100);
            return false;
          }
        }
        break;
      case "s":
        if (e.ctrlKey || e.metaKey) {
          if (!editing) {
            e.preventDefault();
            onSave && onSave(table, {
              pointing: choosing,
              selectingFrom: {
                y: selectingZone.startY,
                x: selectingZone.startX
              },
              selectingTo: {
                y: selectingZone.endY,
                x: selectingZone.endX
              }
            });
            return false;
          }
        }
        break;
      case "v":
        if (e.ctrlKey || e.metaKey) {
          if (!editing) {
            window.setTimeout(() => {
              dispatch(paste({ text: input.value }));
              dispatch(setInputting(""));
            }, 50);
            return false;
          }
        }
        break;
      case "x":
        if (e.ctrlKey || e.metaKey) {
          if (!editing) {
            e.preventDefault();
            const area = clip(store);
            dispatch(cut(areaToZone(area)));
            input.focus();
            return false;
          }
        }
        break;
      case "z":
        if (e.ctrlKey || e.metaKey) {
          if (!editing) {
            e.preventDefault();
            if (e.shiftKey) {
              dispatch(redo(null));
            } else {
              dispatch(undo(null));
            }
            return false;
          }
        }
        break;
      case ";":
        if (e.ctrlKey || e.metaKey) {
          if (!editing) {
            e.preventDefault();
            writeCell((/* @__PURE__ */ new Date()).toDateString());
          }
        }
        break;
    }
    if (e.ctrlKey || e.metaKey) {
      return false;
    }
    if (isPrevented(cell == null ? void 0 : cell.prevention, Write)) {
      console.warn("This cell is protected from writing.");
      return false;
    }
    dispatch(setEditingCell(address));
    if (!editing) {
      dispatch(setInputting(""));
    }
    return false;
  };
  return /* @__PURE__ */ jsxs(
    Fixed,
    {
      className: `gs-editor ${editing ? "gs-editing" : ""}`,
      style: editing ? { top, left, height } : {},
      ...{
        "data-mode": mode,
        "data-sheet-id": sheetId
      },
      children: [
        showAddress && /* @__PURE__ */ jsx("div", { className: "gs-cell-label", children: address }),
        /* @__PURE__ */ jsxs("div", { className: "gs-editor-inner", style: { width }, children: [
          /* @__PURE__ */ jsx(
            "pre",
            {
              className: "gs-editor-hl",
              style: {
                //...cell?.style,
                height: (_a = editorRef.current) == null ? void 0 : _a.scrollHeight,
                width: (((_b = editorRef.current) == null ? void 0 : _b.scrollWidth) ?? 0) - 4
              },
              children: (cell == null ? void 0 : cell.disableFormula) ? inputting : editorStyle(inputting)
            }
          ),
          /* @__PURE__ */ jsx(
            "textarea",
            {
              autoFocus: true,
              spellCheck: false,
              draggable: false,
              ref: editorRef,
              rows: numLines,
              onFocus: () => {
                var _a2;
                dispatch(setLastFocusedRef(editorRef));
                (_a2 = sheetContext == null ? void 0 : sheetContext.setLastFocusedRef) == null ? void 0 : _a2.call(sheetContext, editorRef);
              },
              style: { minWidth: width, minHeight: height },
              onDoubleClick: (e) => {
                if (isPrevented(cell == null ? void 0 : cell.prevention, Write)) {
                  console.warn("This cell is protected from writing.");
                  return;
                }
                const input = e.currentTarget;
                resetSize(input);
                if (!editing) {
                  dispatch(setInputting(valueString));
                  dispatch(setEditingCell(address));
                  window.setTimeout(() => {
                    input.style.width = `${input.scrollWidth}px`;
                    input.style.height = `${input.scrollHeight}px`;
                    const length = new String(valueString).length;
                    input.setSelectionRange(length, length);
                  }, 20);
                }
              },
              onBlur: (e) => {
                dispatch(setLastEdited(before));
                if (e.target.value.startsWith("=")) {
                  return true;
                } else {
                  if (editing) {
                    writeCell(e.target.value);
                  }
                }
                resetSize(e.target);
              },
              value: inputting,
              onChange: (e) => {
                const input = e.currentTarget;
                expandInput(input);
                dispatch(setInputting(e.currentTarget.value));
              },
              onKeyDown: handleKeyDown,
              onKeyUp: (e) => {
                e.currentTarget;
                const selectingArea = zoneToArea(store.selectingZone);
                handleKeyUp == null ? void 0 : handleKeyUp(e, {
                  pointing: choosing,
                  selectingFrom: { y: selectingArea.top, x: selectingArea.left },
                  selectingTo: { y: selectingArea.bottom, x: selectingArea.right }
                });
              }
            }
          )
        ] })
      ]
    }
  );
};
const editorStyle = (text) => {
  if (text[0] !== "=") {
    return /* @__PURE__ */ jsx(Fragment, { children: text });
  }
  const lexer = new Lexer(text.substring(1));
  lexer.tokenize();
  let palletIndex = 0;
  const exists = {};
  return /* @__PURE__ */ jsxs(Fragment, { children: [
    "=",
    lexer.tokens.map((token, i) => {
      if (token.type === "REF" || token.type === "RANGE") {
        const normalizedToken = token.stringify();
        const existsIndex = exists[normalizedToken];
        if (existsIndex !== void 0) {
          return /* @__PURE__ */ jsx("span", { style: { color: REF_PALETTE[existsIndex % REF_PALETTE.length] }, children: token.stringify() }, i);
        }
        const color = REF_PALETTE[palletIndex % REF_PALETTE.length];
        exists[normalizedToken] = palletIndex++;
        return /* @__PURE__ */ jsx("span", { style: { color }, className: `gs-token-type-${token.type}`, children: normalizedToken }, i);
      }
      return /* @__PURE__ */ jsx("span", { className: `gs-token-type-${token.type} gs-token-entity-type-${typeof token.entity}`, children: token.stringify() }, i);
    })
  ] });
};
const resetSize = (input) => {
  input.style.width = "0px";
  input.style.height = "0px";
};
export {
  Editor,
  editorStyle
};
//# sourceMappingURL=Editor.js.map
