import { jsxs, jsx } from "react/jsx-runtime";
import React__default from "react";
import { Context } from "../store/index.js";
import { p2a } from "../lib/converters.js";
import { setEditingCell, setInputting, walk, setLastEdited, setLastFocusedRef, write } from "../store/actions.js";
import { isPrevented, Write } from "../lib/prevention.js";
import { insertTextAtCursor } from "../lib/input.js";
import { useSheetContext } from "./SheetProvider.js";
import { editorStyle } from "./Editor.js";
const FormulaBar = () => {
  var _a, _b;
  const { store, dispatch } = React__default.useContext(Context);
  const [before, setBefore] = React__default.useState("");
  const { choosing, editorRef, largeEditorRef, table, inputting, editingCell } = store;
  const [, sheetContext] = useSheetContext();
  const hlRef = React__default.useRef(null);
  const address = choosing.x === -1 ? "" : p2a(choosing);
  const cell = table.getByPoint(choosing);
  React__default.useEffect(() => {
    var _a2;
    let value = ((_a2 = table.getByPoint(choosing)) == null ? void 0 : _a2.value) ?? "";
    value = table.stringify(choosing, value);
    largeEditorRef.current.value = value;
    setBefore(value);
  }, [address, table]);
  const writeCell = (value) => {
    if (before !== value) {
      dispatch(write(value));
    }
    dispatch(setEditingCell(""));
    editorRef.current.focus();
  };
  React__default.useEffect(() => {
    const observer = new ResizeObserver((entries) => {
      entries.forEach(updateScroll);
    });
    if (largeEditorRef.current) {
      observer.observe(largeEditorRef.current);
    }
    return () => {
      observer.disconnect();
    };
  }, []);
  const largeInput = largeEditorRef.current;
  const handleFocus = () => {
    var _a2;
    if (!largeInput) {
      return;
    }
    dispatch(setEditingCell(address));
    dispatch(setLastFocusedRef(largeEditorRef));
    (_a2 = sheetContext == null ? void 0 : sheetContext.setLastFocusedRef) == null ? void 0 : _a2.call(sheetContext, largeEditorRef);
  };
  const handleInput = (e) => {
    var _a2;
    dispatch(setInputting(e.currentTarget.value));
    (_a2 = sheetContext == null ? void 0 : sheetContext.forceRender) == null ? void 0 : _a2.call(sheetContext);
  };
  const updateScroll = () => {
    if (!hlRef.current || !largeEditorRef.current) {
      return;
    }
    hlRef.current.style.height = `${largeEditorRef.current.clientHeight}px`;
    hlRef.current.scrollLeft = largeEditorRef.current.scrollLeft;
    hlRef.current.scrollTop = largeEditorRef.current.scrollTop;
  };
  return /* @__PURE__ */ jsxs("label", { className: "gs-formula-bar", children: [
    /* @__PURE__ */ jsx("div", { className: "gs-selecting-address", children: address }),
    /* @__PURE__ */ jsx("div", { className: "gs-fx", children: "Fx" }),
    /* @__PURE__ */ jsxs("div", { className: "gs-formula-bar-editor-inner", children: [
      /* @__PURE__ */ jsx(
        "div",
        {
          className: "gs-editor-hl",
          ref: hlRef,
          style: {
            height: (_a = largeEditorRef.current) == null ? void 0 : _a.clientHeight,
            width: (_b = largeEditorRef.current) == null ? void 0 : _b.clientWidth
          },
          children: (cell == null ? void 0 : cell.disableFormula) ? inputting : editorStyle(inputting)
        }
      ),
      /* @__PURE__ */ jsx(
        "textarea",
        {
          rows: 1,
          spellCheck: false,
          ref: largeEditorRef,
          value: inputting,
          onInput: handleInput,
          onFocus: handleFocus,
          onBlur: (e) => {
            dispatch(setLastEdited(before));
            if (e.target.value.startsWith("=")) {
              return true;
            } else {
              if (editingCell) {
                writeCell(e.target.value);
              }
            }
          },
          onKeyDown: (e) => {
            const input = e.currentTarget;
            switch (e.key) {
              case "Enter": {
                if (e.altKey) {
                  insertTextAtCursor(input, "\n");
                } else {
                  writeCell(input.value);
                  dispatch(setInputting(""));
                  dispatch(
                    walk({
                      numRows: table.getNumRows(),
                      numCols: table.getNumCols(),
                      deltaY: 1,
                      deltaX: 0
                    })
                  );
                  e.preventDefault();
                  return false;
                }
                break;
              }
              case "Escape": {
                input.value = before;
                dispatch(setEditingCell(""));
                e.preventDefault();
                editorRef.current.focus();
                break;
              }
            }
            const cell2 = table.getByPoint(choosing);
            if (isPrevented(cell2 == null ? void 0 : cell2.prevention, Write)) {
              console.warn("This cell is protected from writing.");
              e.preventDefault();
            }
            updateScroll();
            return false;
          },
          onKeyUp: updateScroll,
          onScroll: updateScroll
        }
      )
    ] })
  ] });
};
export {
  FormulaBar
};
//# sourceMappingURL=FormulaBar.js.map
