import { jsx, jsxs } from "react/jsx-runtime";
import * as React from "react";
import { HEADER_HEIGHT, HEADER_WIDTH, HISTORY_LIMIT, DEFAULT_HEIGHT, SHEET_HEIGHT, DEFAULT_WIDTH, SHEET_WIDTH } from "../constants.js";
import { functions } from "../formula/mapping.js";
import { Context } from "../store/index.js";
import { reducer } from "../store/actions.js";
import { Editor } from "./Editor.js";
import { StoreInitializer } from "./StoreInitializer.js";
import { Resizer } from "./Resizer.js";
import { Emitter } from "./Emitter.js";
import { ContextMenu } from "./ContextMenu.js";
import { Table } from "../lib/table.js";
import { Tabular } from "./Tabular.js";
import { getMaxSizesFromCells } from "../lib/structs.js";
import { x2c, y2r } from "../lib/converters.js";
import { embedStyle } from "../styles/embedder.js";
import { useSheetContext } from "./SheetProvider.js";
import { FormulaBar } from "./FormulaBar.js";
import { SearchBar } from "./SearchBar.js";
function GridSheet({
  initialCells,
  sheetName = "",
  tableRef,
  options = {},
  className,
  style,
  additionalFunctions = {}
}) {
  const { sheetResize, showFormulaBar = true, onInit, mode = "light" } = options;
  const [prevSheetName, setPrevSheetName] = React.useState(sheetName);
  const rootRef = React.useRef(null);
  const mainRef = React.useRef(null);
  const searchInputRef = React.useRef(null);
  const editorRef = React.useRef(null);
  const largeEditorRef = React.useRef(null);
  const tabularRef = React.useRef(null);
  const lastFocusedRef = React.useRef(null);
  const [sheetProvided, sheetContext] = useSheetContext();
  const [initialState] = React.useState(() => {
    const {
      headerHeight = HEADER_HEIGHT,
      headerWidth = HEADER_WIDTH,
      historyLimit = HISTORY_LIMIT,
      renderers,
      parsers,
      labelers,
      minNumRows,
      maxNumRows,
      minNumCols,
      maxNumCols
    } = options;
    const table = new Table({
      historyLimit,
      parsers,
      renderers,
      labelers,
      minNumRows,
      maxNumRows,
      minNumCols,
      maxNumCols,
      headerHeight,
      headerWidth,
      sheetName,
      functions: { ...functions, ...additionalFunctions }
    });
    let sheetId = 0;
    if (sheetProvided) {
      sheetId = sheetContext.head.current++;
      table.tables = sheetContext.tables.current;
      table.sheets = sheetContext.sheets.current;
    }
    table.tables[sheetId] = table;
    table.sheetId = sheetId;
    table.initialize(initialCells);
    onInit == null ? void 0 : onInit(table);
    return {
      sheetId,
      table,
      // temporary (see StoreInitializer for detail)
      tableInitialized: false,
      rootRef,
      mainRef,
      searchInputRef,
      editorRef,
      largeEditorRef,
      tabularRef,
      lastFocusedRef,
      choosing: { y: 1, x: 1 },
      cutting: false,
      inputting: "",
      selectingZone: { startY: -1, startX: -1, endY: -1, endX: -1 },
      copyingZone: { startY: -1, startX: -1, endY: -1, endX: -1 },
      autofillDraggingTo: null,
      leftHeaderSelecting: false,
      topHeaderSelecting: false,
      editingCell: "",
      editorRect: { y: 0, x: 0, height: 0, width: 0 },
      resizingRect: { y: -1, x: -1, height: -1, width: -1 },
      sheetHeight: 0,
      sheetWidth: 0,
      headerHeight: 0,
      headerWidth: 0,
      entering: false,
      matchingCells: [],
      matchingCellIndex: 0,
      searchCaseSensitive: false,
      editingOnEnter: true,
      showAddress: true,
      contextMenuPosition: { y: -1, x: -1 },
      resizingPositionY: [-1, -1, -1],
      resizingPositionX: [-1, -1, -1],
      minNumRows: 1,
      maxNumRows: -1,
      minNumCols: 1,
      maxNumCols: -1,
      mode: "light",
      lastEdited: ""
    };
  });
  const [store, dispatch] = React.useReducer(
    reducer,
    initialState,
    () => initialState
  );
  React.useEffect(() => {
    embedStyle();
  }, []);
  React.useEffect(() => {
    var _a;
    if (!sheetProvided) {
      return;
    }
    sheetContext.tables.current[store.sheetId] = store.table;
    (_a = sheetContext == null ? void 0 : sheetContext.forceRender) == null ? void 0 : _a.call(sheetContext);
  }, [store.table]);
  React.useEffect(() => {
    if (!sheetProvided) {
      return;
    }
    if (prevSheetName !== sheetName) {
      delete sheetContext.sheets.current[prevSheetName];
      setPrevSheetName(sheetName);
    }
    if (sheetName) {
      sheetContext.sheets.current[sheetName] = store.sheetId;
    }
    store.table.sheetName = sheetName;
  }, [sheetName]);
  const [sheetHeight, setSheetHeight] = React.useState(
    (options == null ? void 0 : options.sheetHeight) || estimateSheetHeight({ options, initialData: initialCells })
  );
  const [sheetWidth, setSheetWidth] = React.useState(
    (options == null ? void 0 : options.sheetWidth) || estimateSheetWidth({ options, initialData: initialCells })
  );
  React.useEffect(() => {
    const intervalId = window.setInterval(() => {
      var _a, _b;
      setSheetHeight(((_a = mainRef.current) == null ? void 0 : _a.clientHeight) || 0);
      setSheetWidth(((_b = mainRef.current) == null ? void 0 : _b.clientWidth) || 0);
    }, 1e3);
    return () => window.clearInterval(intervalId);
  }, []);
  React.useEffect(() => {
    if (options.sheetHeight) {
      setSheetHeight(options.sheetHeight);
    }
  }, [options.sheetHeight]);
  React.useEffect(() => {
    if (options.sheetWidth) {
      setSheetWidth(options.sheetWidth);
    }
  }, [options.sheetWidth]);
  const { onChange, onSelect, onKeyUp } = options;
  return /* @__PURE__ */ jsx(Context.Provider, { value: { store, dispatch }, children: /* @__PURE__ */ jsxs(
    "div",
    {
      className: `gs-root1`,
      ref: rootRef,
      "data-sheet-name": sheetName,
      "data-mode": mode,
      style: { maxWidth: `min(100%, ${store.table.totalWidth + 2}px)` },
      children: [
        typeof store.searchQuery === "undefined" ? showFormulaBar && /* @__PURE__ */ jsx(FormulaBar, {}) : /* @__PURE__ */ jsx(SearchBar, {}),
        /* @__PURE__ */ jsxs(
          "div",
          {
            className: `gs-main ${className || ""}`,
            ref: mainRef,
            style: {
              maxWidth: `min(100%-1px, ${store.table.totalWidth + 2}px)`,
              maxHeight: store.table.totalHeight + 2,
              ...style,
              resize: sheetResize
            },
            children: [
              /* @__PURE__ */ jsx(Editor, { mode, handleKeyUp: onKeyUp }),
              /* @__PURE__ */ jsx(Tabular, { tableRef }),
              /* @__PURE__ */ jsx(
                StoreInitializer,
                {
                  initialCells,
                  options: { ...options, sheetHeight, sheetWidth },
                  additionalFunctions
                }
              ),
              /* @__PURE__ */ jsx(ContextMenu, {}),
              /* @__PURE__ */ jsx(Resizer, {}),
              /* @__PURE__ */ jsx(Emitter, { onChange, onSelect })
            ]
          }
        )
      ]
    }
  ) });
}
const estimateSheetHeight = ({ initialData, options }) => {
  var _a, _b;
  const auto = getMaxSizesFromCells(initialData);
  let estimatedHeight = options.headerHeight || HEADER_HEIGHT;
  for (let y = 0; y < auto.numRows; y++) {
    const row = y2r(y);
    const height = ((_a = initialData == null ? void 0 : initialData[row]) == null ? void 0 : _a.height) || ((_b = initialData == null ? void 0 : initialData.default) == null ? void 0 : _b.height) || DEFAULT_HEIGHT;
    if (estimatedHeight + height > SHEET_HEIGHT) {
      return SHEET_HEIGHT;
    }
    estimatedHeight += height;
  }
  return estimatedHeight + 3;
};
const estimateSheetWidth = ({ initialData, options }) => {
  var _a, _b;
  const auto = getMaxSizesFromCells(initialData);
  let estimatedWidth = options.headerWidth || HEADER_WIDTH;
  for (let x = 0; x < auto.numCols; x++) {
    const col = x2c(x);
    const width = ((_a = initialData == null ? void 0 : initialData[col]) == null ? void 0 : _a.width) || ((_b = initialData == null ? void 0 : initialData.default) == null ? void 0 : _b.width) || DEFAULT_WIDTH;
    if (estimatedWidth + width > SHEET_WIDTH) {
      return SHEET_WIDTH;
    }
    estimatedWidth += width;
  }
  return estimatedWidth + 3;
};
export {
  GridSheet
};
//# sourceMappingURL=GridSheet.js.map
