import { jsx, jsxs } from "react/jsx-runtime";
import React__default from "react";
import { y2r } from "../lib/converters.js";
import { zoneToArea, between } from "../lib/structs.js";
import { Context } from "../store/index.js";
import { setResizingPositionY, drag, select, selectRows, choose, setContextMenuPosition, setEditingCell } from "../store/actions.js";
import { DEFAULT_HEIGHT, DUMMY_IMG } from "../constants.js";
import { isPrevented, Resize } from "../lib/prevention.js";
import { insertRef, isRefInsertable } from "../lib/input.js";
import { useSheetContext } from "./SheetProvider.js";
const HeaderCellLeft = React__default.memo(({ y }) => {
  const rowId = `${y2r(y)}`;
  const { store, dispatch } = React__default.useContext(Context);
  const {
    choosing,
    selectingZone,
    leftHeaderSelecting,
    resizingRect,
    headerWidth,
    editorRef,
    table,
    autofillDraggingTo
  } = store;
  const row = table.getByPoint({ y, x: 0 });
  const height = (row == null ? void 0 : row.height) || DEFAULT_HEIGHT;
  const [sheetProvided, sheetContext] = useSheetContext();
  const differentSheetFocused = sheetProvided && (sheetContext == null ? void 0 : sheetContext.lastFocusedRef) !== store.lastFocusedRef;
  const lastFocusedRef = (sheetContext == null ? void 0 : sheetContext.lastFocusedRef) || store.lastFocusedRef;
  const lastInput = lastFocusedRef.current;
  return /* @__PURE__ */ jsx(
    "th",
    {
      "data-y": y,
      className: `gs-th gs-th-left ${choosing.y === y ? "gs-choosing" : ""} ${between({ start: selectingZone.startY, end: selectingZone.endY }, y) ? leftHeaderSelecting ? "gs-th-selecting" : "gs-selecting" : ""}`,
      style: { height },
      onClick: (e) => {
        let startY = e.shiftKey ? selectingZone.startY : y;
        if (startY === -1) {
          startY = choosing.y;
        }
        const fullColId = `${table.sheetPrefix(!differentSheetFocused)}${rowId}:${rowId}`;
        const inserted = insertRef(lastInput, fullColId);
        if (inserted) {
          return false;
        }
        dispatch(
          selectRows({
            range: { start: startY, end: y },
            numCols: table.getNumCols()
          })
        );
        dispatch(setContextMenuPosition({ y: -1, x: -1 }));
        dispatch(choose({ y: startY, x: 1 }));
        dispatch(setEditingCell(""));
        editorRef.current.focus();
        return false;
      },
      onContextMenu: (e) => {
        e.preventDefault();
        dispatch(setContextMenuPosition({ y: e.clientY, x: e.clientX }));
        return false;
      },
      children: /* @__PURE__ */ jsx(
        "div",
        {
          className: "gs-th-inner-wrap",
          draggable: true,
          onDragStart: (e) => {
            e.dataTransfer.setDragImage(DUMMY_IMG, 0, 0);
            const insertable = isRefInsertable(lastInput);
            if (insertable) {
              dispatch(select({ startY: y, startX: table.getNumCols(), endY: y, endX: 0 }));
              return false;
            }
            dispatch(
              selectRows({
                range: { start: y, end: y },
                numCols: table.getNumCols()
              })
            );
            dispatch(choose({ y, x: 1 }));
            return false;
          },
          onDragEnd: () => {
            if (isRefInsertable(lastInput)) {
              dispatch(select({ startY: -1, startX: -1, endY: -1, endX: -1 }));
            }
          },
          onDragEnter: () => {
            const newArea = zoneToArea({ ...selectingZone, endY: y, endX: 1 });
            const [top, bottom] = [y2r(newArea.top), y2r(newArea.bottom)];
            const fullRange = `${table.sheetPrefix(!differentSheetFocused)}${top}:${bottom}`;
            insertRef(lastInput, fullRange);
            if (resizingRect.y === -1 && autofillDraggingTo == null) {
              const { startX } = selectingZone;
              if (startX === 1) {
                dispatch(drag({ y, x: table.getNumCols() }));
              } else {
                dispatch(drag({ y, x: 1 }));
              }
            }
            return false;
          },
          onDragOver: (e) => {
            e.dataTransfer.dropEffect = "move";
            e.preventDefault();
          },
          children: /* @__PURE__ */ jsxs("div", { className: "gs-th-inner", style: { width: headerWidth, position: "relative" }, children: [
            (row == null ? void 0 : row.labeler) ? table.getLabel(row.labeler, y) : rowId,
            /* @__PURE__ */ jsx(
              "div",
              {
                className: `gs-resizer ${isPrevented(row == null ? void 0 : row.prevention, Resize) ? "gs-protected" : ""}`,
                style: { width: headerWidth },
                onMouseDown: (e) => {
                  dispatch(setResizingPositionY([y, e.clientY, e.clientY]));
                  e.preventDefault();
                  e.stopPropagation();
                }
              }
            )
          ] })
        }
      )
    }
  );
});
export {
  HeaderCellLeft
};
//# sourceMappingURL=HeaderCellLeft.js.map
