import { jsx, jsxs } from "react/jsx-runtime";
import React__default from "react";
import { x2c } from "../lib/converters.js";
import { zoneToArea, between } from "../lib/structs.js";
import { Context } from "../store/index.js";
import { setResizingPositionX, drag, select, selectCols, choose, setContextMenuPosition, setEditingCell } from "../store/actions.js";
import { DEFAULT_WIDTH, DUMMY_IMG } from "../constants.js";
import { isPrevented, Resize } from "../lib/prevention.js";
import { useSheetContext } from "./SheetProvider.js";
import { insertRef, isRefInsertable } from "../lib/input.js";
const HeaderCellTop = React__default.memo(({ x }) => {
  const { store, dispatch } = React__default.useContext(Context);
  const colId = x2c(x);
  const {
    table,
    choosing,
    selectingZone,
    resizingRect,
    topHeaderSelecting,
    headerHeight,
    editorRef,
    autofillDraggingTo
  } = store;
  const col = table.getByPoint({ y: 0, x });
  const width = (col == null ? void 0 : col.width) || DEFAULT_WIDTH;
  const [sheetProvided, sheetContext] = useSheetContext();
  const differentSheetFocused = sheetProvided && (sheetContext == null ? void 0 : sheetContext.lastFocusedRef) !== store.lastFocusedRef;
  const lastFocusedRef = (sheetContext == null ? void 0 : sheetContext.lastFocusedRef) || store.lastFocusedRef;
  const lastInput = lastFocusedRef.current;
  return /* @__PURE__ */ jsx(
    "th",
    {
      "data-x": x,
      className: `gs-th gs-th-top ${choosing.x === x ? "gs-choosing" : ""} ${between({ start: selectingZone.startX, end: selectingZone.endX }, x) ? topHeaderSelecting ? "gs-th-selecting" : "gs-selecting" : ""}`,
      style: { width, minWidth: width, maxWidth: width },
      onClick: (e) => {
        let startX = e.shiftKey ? selectingZone.startX : x;
        if (startX === -1) {
          startX = choosing.x;
        }
        const fullColId = `${table.sheetPrefix(!differentSheetFocused)}${colId}:${colId}`;
        const inserted = insertRef(lastInput, fullColId);
        if (inserted) {
          return false;
        }
        dispatch(
          selectCols({
            range: { start: startX, end: x },
            numRows: table.getNumRows()
          })
        );
        dispatch(setContextMenuPosition({ y: -1, x: -1 }));
        dispatch(choose({ y: 1, x: startX }));
        dispatch(setEditingCell(""));
        editorRef.current.focus();
        return false;
      },
      onContextMenu: (e) => {
        e.preventDefault();
        dispatch(setContextMenuPosition({ y: e.clientY, x: e.clientX }));
        return false;
      },
      children: /* @__PURE__ */ jsx(
        "div",
        {
          className: "gs-th-inner-wrap",
          draggable: true,
          onDragStart: (e) => {
            e.dataTransfer.setDragImage(DUMMY_IMG, 0, 0);
            const insertable = isRefInsertable(lastInput);
            if (insertable) {
              dispatch(select({ startY: table.getNumRows(), startX: x, endY: 0, endX: x }));
              return false;
            }
            dispatch(
              selectCols({
                range: { start: x, end: x },
                numRows: table.getNumRows()
              })
            );
            dispatch(choose({ y: 1, x }));
            return false;
          },
          onDragEnd: () => {
            if (isRefInsertable(lastInput)) {
              dispatch(select({ startY: -1, startX: -1, endY: -1, endX: -1 }));
            }
          },
          onDragEnter: () => {
            const newArea = zoneToArea({ ...selectingZone, endY: 1, endX: x });
            const [left, right] = [x2c(newArea.left), x2c(newArea.right)];
            const fullRange = `${table.sheetPrefix(!differentSheetFocused)}${left}:${right}`;
            insertRef(lastInput, fullRange);
            if (resizingRect.x === -1 && autofillDraggingTo == null) {
              const { startY } = selectingZone;
              if (startY === 1) {
                dispatch(drag({ y: table.getNumRows(), x }));
              } else {
                dispatch(drag({ y: 1, x }));
              }
            }
            return false;
          },
          onDragOver: (e) => {
            e.dataTransfer.dropEffect = "move";
            e.preventDefault();
          },
          children: /* @__PURE__ */ jsxs("div", { className: "gs-th-inner", style: { height: headerHeight, position: "relative" }, children: [
            (col == null ? void 0 : col.labeler) ? table.getLabel(col.labeler, x) : colId,
            /* @__PURE__ */ jsx(
              "div",
              {
                className: `gs-resizer ${isPrevented(col == null ? void 0 : col.prevention, Resize) ? "gs-protected" : ""}`,
                style: { height: headerHeight },
                onMouseDown: (e) => {
                  dispatch(setResizingPositionX([x, e.clientX, e.clientX]));
                  e.preventDefault();
                  e.stopPropagation();
                },
                children: /* @__PURE__ */ jsx("i", {})
              }
            )
          ] })
        }
      )
    }
  );
});
export {
  HeaderCellTop
};
//# sourceMappingURL=HeaderCellTop.js.map
