import { jsxs, jsx } from "react/jsx-runtime";
import React__default from "react";
import { Context } from "../store/index.js";
import { setResizingPositionY, setResizingPositionX, updateTable } from "../store/actions.js";
import { DEFAULT_WIDTH, DEFAULT_HEIGHT, MIN_HEIGHT, MIN_WIDTH } from "../constants.js";
import { zoneToArea, between, makeSequence } from "../lib/structs.js";
import { p2a } from "../lib/converters.js";
const Resizer = React__default.memo(() => {
  const { store, dispatch } = React__default.useContext(Context);
  const {
    resizingPositionY: posY,
    resizingPositionX: posX,
    table,
    leftHeaderSelecting,
    topHeaderSelecting,
    selectingZone,
    editorRef,
    mainRef
  } = store;
  const [y, startY, endY] = posY;
  const [x, startX, endX] = posX;
  if (y === -1 && x === -1) {
    return null;
  }
  if (mainRef.current == null || editorRef.current == null) {
    return null;
  }
  const cell = table.getByPoint({ y: y === -1 ? 0 : y, x: x === -1 ? 0 : x });
  const { y: offsetY, x: offsetX } = mainRef.current.getBoundingClientRect();
  const baseWidth = (cell == null ? void 0 : cell.width) || DEFAULT_WIDTH;
  const baseHeight = (cell == null ? void 0 : cell.height) || DEFAULT_HEIGHT;
  const width = baseWidth + (endX - startX);
  const height = baseHeight + (endY - startY);
  const handleResizeEnd = () => {
    const selectingArea = zoneToArea(selectingZone);
    const { top, left, bottom, right } = selectingArea;
    const diff = {};
    if (x !== -1) {
      let xs = [x];
      if (topHeaderSelecting && between({ start: left, end: right }, x)) {
        xs = makeSequence(left, right + 1);
      }
      xs.forEach((x2) => {
        diff[p2a({ y: 0, x: x2 })] = { width };
      });
    }
    if (y !== -1) {
      let ys = [y];
      if (leftHeaderSelecting && between({ start: top, end: bottom }, y)) {
        ys = makeSequence(top, bottom + 1);
      }
      ys.forEach((y2) => {
        diff[p2a({ y: y2, x: 0 })] = { height };
      });
    }
    const newTable = table.update({
      diff,
      partial: true,
      operator: "USER",
      reflection: { selectingZone }
    });
    dispatch(updateTable(newTable));
    dispatch(setResizingPositionY([-1, -1, -1]));
    dispatch(setResizingPositionX([-1, -1, -1]));
    editorRef.current.focus();
  };
  const handleResizeMove = (e) => {
    if (y !== -1) {
      let endY2 = e.clientY;
      const height2 = baseHeight + (endY2 - startY);
      if (height2 < MIN_HEIGHT) {
        endY2 += MIN_HEIGHT - height2;
      }
      dispatch(setResizingPositionY([y, startY, endY2]));
    } else if (x !== -1) {
      let endX2 = e.clientX;
      const width2 = baseWidth + (endX2 - startX);
      if (width2 < MIN_WIDTH) {
        endX2 += MIN_WIDTH - width2;
      }
      dispatch(setResizingPositionX([x, startX, endX2]));
    }
  };
  return /* @__PURE__ */ jsxs("div", { className: "gs-resizing", onMouseUp: handleResizeEnd, onMouseMove: handleResizeMove, children: [
    x !== -1 && /* @__PURE__ */ jsx("div", { className: "gs-line", style: { width: 1, height: "100%", left: endX - offsetX }, children: /* @__PURE__ */ jsxs("span", { style: { left: "-50%" }, children: [
      width,
      "px"
    ] }) }),
    y !== -1 && /* @__PURE__ */ jsx("div", { className: "gs-line", style: { width: "100%", height: 1, top: endY - offsetY }, children: /* @__PURE__ */ jsxs("span", { style: { top: "-50%" }, children: [
      height,
      "px"
    ] }) })
  ] });
});
export {
  Resizer
};
//# sourceMappingURL=Resizer.js.map
