import { jsxs, jsx } from "react/jsx-runtime";
import { a2p } from "../lib/converters.js";
import React__default from "react";
import { Context } from "../store/index.js";
import { search, setSearchQuery, setSearchCaseSensitive } from "../store/actions.js";
import { smartScroll } from "../lib/virtualization.js";
import { SearchIcon } from "./svg/SearchIcon.js";
import { CloseIcon } from "./svg/CloseIcon.js";
const SearchBar = () => {
  const { store, dispatch } = React__default.useContext(Context);
  const {
    rootRef,
    editorRef,
    searchInputRef,
    tabularRef,
    searchQuery,
    searchCaseSensitive,
    matchingCellIndex,
    matchingCells,
    table
  } = store;
  const matchingCell = matchingCells[matchingCellIndex];
  React__default.useEffect(() => {
    if (!matchingCell) {
      return;
    }
    const point = a2p(matchingCell);
    if (typeof point === "undefined") {
      return;
    }
    smartScroll(table, tabularRef.current, point);
  }, [searchQuery, matchingCellIndex, searchCaseSensitive]);
  if (typeof searchQuery === "undefined") {
    return null;
  }
  if (rootRef.current === null) {
    return null;
  }
  return /* @__PURE__ */ jsxs("label", { className: `gs-search-bar ${matchingCells.length > 0 ? "gs-search-found" : ""}`, children: [
    /* @__PURE__ */ jsxs(
      "div",
      {
        className: "gs-search-progress",
        onClick: (e) => {
          const input = e.currentTarget.previousSibling;
          (input == null ? void 0 : input.nodeName) === "INPUT" && input.focus();
        },
        children: [
          matchingCells.length === 0 ? 0 : matchingCellIndex + 1,
          " / ",
          matchingCells.length
        ]
      }
    ),
    /* @__PURE__ */ jsx("div", { className: "gs-search-bar-icon", onClick: () => dispatch(search(1)), children: /* @__PURE__ */ jsx(SearchIcon, { style: { verticalAlign: "middle", marginLeft: "5px" } }) }),
    /* @__PURE__ */ jsx(
      "textarea",
      {
        ref: searchInputRef,
        value: searchQuery,
        onChange: (e) => dispatch(setSearchQuery(e.target.value)),
        onKeyDown: (e) => {
          if (e.key === "Escape") {
            const el = editorRef == null ? void 0 : editorRef.current;
            if (el) {
              el.focus();
            }
            dispatch(setSearchQuery(void 0));
          }
          if (e.key === "f" && (e.ctrlKey || e.metaKey)) {
            e.preventDefault();
            return false;
          }
          if (e.key === "Enter") {
            dispatch(search(e.shiftKey ? -1 : 1));
            e.preventDefault();
            return false;
          }
          return true;
        }
      }
    ),
    /* @__PURE__ */ jsx("div", { className: `gs-search-casesensitive`, children: /* @__PURE__ */ jsx(
      "span",
      {
        className: `${searchCaseSensitive ? "gs-search-casesensitive-on" : ""}`,
        onClick: () => dispatch(setSearchCaseSensitive(!searchCaseSensitive)),
        children: "Aa"
      }
    ) }),
    /* @__PURE__ */ jsx(
      "a",
      {
        className: "gs-search-close",
        onClick: () => {
          var _a;
          dispatch(setSearchQuery(void 0));
          (_a = editorRef.current) == null ? void 0 : _a.focus();
        },
        children: /* @__PURE__ */ jsx(CloseIcon, { style: { verticalAlign: "middle" } })
      }
    )
  ] });
};
export {
  SearchBar
};
//# sourceMappingURL=SearchBar.js.map
