import { jsx, Fragment } from "react/jsx-runtime";
import React__default from "react";
import { Context } from "../store/index.js";
import { initializeTable, setSheetHeight, setSheetWidth, setHeaderHeight, setHeaderWidth, setEditingOnEnter, setShowAddress, setMode, setOnSave } from "../store/actions.js";
import { HEADER_HEIGHT, HEADER_WIDTH } from "../constants.js";
import { useSheetContext } from "./SheetProvider.js";
import { usePluginContext } from "./PluginBase.js";
const StoreInitializer = ({ options = {} }) => {
  const {
    headerHeight = HEADER_HEIGHT,
    headerWidth = HEADER_WIDTH,
    sheetHeight,
    sheetWidth,
    editingOnEnter,
    showAddress,
    mode,
    onSave
  } = options;
  const [sheetProvided, sheetContext] = useSheetContext();
  const { store, dispatch } = React__default.useContext(Context);
  React__default.useEffect(() => {
    const { table, tableInitialized } = store;
    if (table == null || tableInitialized) {
      return;
    }
    if (!sheetProvided || sheetContext.mounted) {
      table.absolutizeFormula();
      dispatch(initializeTable(table.shallowCopy()));
    }
  }, [sheetContext.mounted]);
  React__default.useEffect(() => {
    if (sheetHeight) {
      dispatch(setSheetHeight(sheetHeight));
    }
  }, [sheetHeight]);
  React__default.useEffect(() => {
    if (sheetWidth) {
      dispatch(setSheetWidth(sheetWidth));
    }
  }, [sheetWidth]);
  React__default.useEffect(() => {
    if (headerHeight) {
      dispatch(setHeaderHeight(headerHeight));
    }
  }, [headerHeight]);
  React__default.useEffect(() => {
    if (headerWidth) {
      dispatch(setHeaderWidth(headerWidth));
    }
  }, [headerWidth]);
  React__default.useEffect(() => {
    if (typeof editingOnEnter !== "undefined") {
      dispatch(setEditingOnEnter(editingOnEnter));
    }
  }, [editingOnEnter]);
  React__default.useEffect(() => {
    if (typeof showAddress !== "undefined") {
      dispatch(setShowAddress(showAddress));
    }
  }, [showAddress]);
  React__default.useEffect(() => {
    if (mode) {
      dispatch(setMode(mode));
    }
  }, [mode]);
  React__default.useEffect(() => {
    if (typeof onSave !== "undefined") {
      dispatch(setOnSave(onSave));
    }
  }, [onSave]);
  const [pluginProvided, pluginContext] = usePluginContext();
  React__default.useEffect(() => {
    if (!pluginProvided) {
      return;
    }
    pluginContext.setStore(store);
    pluginContext.setDispatch(() => dispatch);
  }, [store, dispatch]);
  return /* @__PURE__ */ jsx(Fragment, {});
};
export {
  StoreInitializer
};
//# sourceMappingURL=StoreInitializer.js.map
