import { jsx, Fragment, jsxs } from "react/jsx-runtime";
import React__default from "react";
import { Cell } from "./Cell.js";
import { HeaderCellTop } from "./HeaderCellTop.js";
import { HeaderCellLeft } from "./HeaderCellLeft.js";
import { Context } from "../store/index.js";
import { updateTable, setInputting, choose, select, setEntering } from "../store/actions.js";
import { virtualize } from "../lib/virtualization.js";
import { stripAddressAbsolute, p2a, a2p } from "../lib/converters.js";
import { zoneToArea } from "../lib/structs.js";
import { Lexer, stripSheetName } from "../formula/evaluator.js";
import { REF_PALETTE } from "../lib/palette.js";
import { useSheetContext } from "./SheetProvider.js";
import { Autofill } from "../lib/autofill.js";
const createTableRef = () => React__default.useRef(null);
const Tabular = ({ tableRef }) => {
  var _a, _b, _c, _d, _e, _f, _g, _h, _i;
  const [refs, setRefs] = React__default.useState({});
  const [, { externalRefs = {}, setExternalRefs }] = useSheetContext();
  const { store, dispatch } = React__default.useContext(Context);
  const {
    sheetHeight,
    table,
    tableInitialized,
    tabularRef,
    mainRef,
    headerWidth,
    headerHeight,
    editingCell,
    inputting
  } = store;
  const [sheetProvided, sheetContext] = useSheetContext();
  React__default.useEffect(() => {
    const formulaEditing = editingCell && inputting.startsWith("=");
    if (!formulaEditing) {
      setRefs({});
      setExternalRefs == null ? void 0 : setExternalRefs({});
      return;
    }
    const refs2 = {};
    const externalRefs2 = {};
    const lexer = new Lexer(inputting.substring(1));
    lexer.tokenize();
    let i = 0;
    for (const token of lexer.tokens) {
      if (token.type === "REF" || token.type === "RANGE") {
        const normalizedRef = stripAddressAbsolute(token.stringify());
        const splitterIndex = normalizedRef.indexOf("!");
        if (splitterIndex !== -1) {
          const sheetName = normalizedRef.substring(0, splitterIndex);
          const ref = normalizedRef.substring(splitterIndex + 1);
          const stripped = stripSheetName(sheetName);
          const upperRef = ref.toUpperCase();
          if (externalRefs2[stripped] == null) {
            externalRefs2[stripped] = {};
          }
          if (externalRefs2[stripped][upperRef] == null) {
            externalRefs2[stripped][upperRef] = i++;
          }
        } else {
          const upperRef = normalizedRef.toUpperCase();
          if (refs2[upperRef] == null) {
            refs2[upperRef] = i++;
          }
        }
      }
    }
    setRefs(refs2);
    setExternalRefs == null ? void 0 : setExternalRefs(externalRefs2);
  }, [store.inputting, store.editingCell]);
  React__default.useEffect(() => {
    if (tableRef && tableInitialized) {
      tableRef.current = {
        table,
        dispatch: (table2) => {
          dispatch(updateTable(table2));
        }
      };
    }
  }, [table]);
  React__default.useEffect(() => {
    const v = table.stringify(store.choosing);
    dispatch(setInputting(v || ""));
  }, [table, store.choosing]);
  React__default.useEffect(() => {
    var _a2;
    (_a2 = sheetContext == null ? void 0 : sheetContext.setChoosingCell) == null ? void 0 : _a2.call(sheetContext, p2a(store.choosing));
  }, [store.choosing]);
  const [virtualized, setVirtualized] = React__default.useState(null);
  React__default.useEffect(() => {
    setVirtualized(virtualize(table, tabularRef.current));
  }, [tabularRef.current, table, (_a = mainRef.current) == null ? void 0 : _a.clientHeight, (_b = mainRef.current) == null ? void 0 : _b.clientWidth]);
  const operationStyles = useOperationStyles(store, { ...refs, ...externalRefs[table.sheetName] });
  return /* @__PURE__ */ jsx(Fragment, { children: /* @__PURE__ */ jsx(
    "div",
    {
      className: "gs-tabular",
      style: {
        //width: sheetWidth,
        height: sheetHeight
      },
      ref: tabularRef,
      onMouseEnter: () => {
        dispatch(setEntering(true));
      },
      onMouseLeave: () => {
        dispatch(setEntering(false));
      },
      onScroll: (e) => {
        setVirtualized(virtualize(table, e.currentTarget));
      },
      children: /* @__PURE__ */ jsx(
        "div",
        {
          className: "gs-tabular-inner",
          style: {
            width: table.totalWidth + 1,
            height: table.totalHeight + 1
          },
          children: /* @__PURE__ */ jsxs(
            "table",
            {
              className: `gs-table`,
              style: {
                width: table.totalWidth
              },
              children: [
                /* @__PURE__ */ jsx("thead", { className: "gs-thead", style: { height: headerHeight }, children: /* @__PURE__ */ jsxs("tr", { className: "gs-row", children: [
                  /* @__PURE__ */ jsx(
                    "th",
                    {
                      className: "gs-th gs-th-left gs-th-top",
                      style: { position: "sticky", width: headerWidth, height: headerHeight },
                      onClick: () => {
                        dispatch(choose({ y: -1, x: -1 }));
                        window.setTimeout(() => {
                          dispatch(choose({ y: 1, x: 1 }));
                          dispatch(
                            select({
                              startY: 1,
                              startX: 1,
                              endY: table.getNumRows(),
                              endX: table.getNumCols()
                            })
                          );
                        }, 100);
                      },
                      children: /* @__PURE__ */ jsx("div", { className: "gs-th-inner" })
                    }
                  ),
                  /* @__PURE__ */ jsx(
                    "th",
                    {
                      className: "gs-adjuster gs-adjuster-horizontal gs-adjuster-horizontal-left",
                      style: { width: ((_c = virtualized == null ? void 0 : virtualized.adjuster) == null ? void 0 : _c.left) || 1 }
                    }
                  ),
                  (_e = (_d = virtualized == null ? void 0 : virtualized.xs) == null ? void 0 : _d.map) == null ? void 0 : _e.call(_d, (x) => /* @__PURE__ */ jsx(HeaderCellTop, { x }, x)),
                  /* @__PURE__ */ jsx(
                    "th",
                    {
                      className: "gs-adjuster gs-adjuster-horizontal gs-adjuster-horizontal-right",
                      style: { width: (_f = virtualized == null ? void 0 : virtualized.adjuster) == null ? void 0 : _f.right }
                    }
                  )
                ] }) }),
                /* @__PURE__ */ jsx("tbody", { className: "gs-table-body-adjuster", children: /* @__PURE__ */ jsxs("tr", { className: "gs-row", children: [
                  /* @__PURE__ */ jsx(
                    "th",
                    {
                      className: `gs-adjuster gs-adjuster-horizontal gs-adjuster-vertical`,
                      style: { height: ((_g = virtualized == null ? void 0 : virtualized.adjuster) == null ? void 0 : _g.top) || 1 }
                    }
                  ),
                  /* @__PURE__ */ jsx("td", { className: "gs-adjuster gs-adjuster-vertical" }),
                  (_h = virtualized == null ? void 0 : virtualized.xs) == null ? void 0 : _h.map((x) => /* @__PURE__ */ jsx("td", { className: "gs-adjuster gs-adjuster-vertical" }, x)),
                  /* @__PURE__ */ jsx("th", { className: `gs-adjuster gs-adjuster-horizontal gs-adjuster-vertical` })
                ] }) }),
                /* @__PURE__ */ jsx("tbody", { className: "gs-table-body-data", children: (_i = virtualized == null ? void 0 : virtualized.ys) == null ? void 0 : _i.map((y) => {
                  var _a2;
                  return /* @__PURE__ */ jsxs("tr", { className: "gs-row", children: [
                    /* @__PURE__ */ jsx(HeaderCellLeft, { y }),
                    /* @__PURE__ */ jsx("td", { className: "gs-adjuster gs-adjuster-horizontal gs-adjuster-horizontal-left" }),
                    (_a2 = virtualized == null ? void 0 : virtualized.xs) == null ? void 0 : _a2.map((x) => /* @__PURE__ */ jsx(Cell, { y, x, operationStyle: operationStyles[p2a({ y, x })] }, x)),
                    /* @__PURE__ */ jsx("td", { className: "gs-adjuster gs-adjuster-horizontal gs-adjuster-horizontal-right" })
                  ] }, y);
                }) })
              ]
            }
          )
        }
      )
    }
  ) });
};
const BORDER_POINTED = "solid 2px #0077ff";
const BORDER_SELECTED = "solid 1px #0077ff";
const BORDER_CUTTING = "dotted 2px #0077ff";
const BORDER_COPYING = "dashed 2px #0077ff";
const SEARCH_MATCHING_BACKGROUND = "rgba(0,200,100,.2)";
const SEARCH_MATCHING_BORDER = "solid 2px #00aa78";
const AUTOFILL_BORDER = "dashed 1px #444444";
const useOperationStyles = (store, refs) => {
  const cellStyles = {};
  const updateStyle = (point, style) => {
    const address = p2a(point);
    cellStyles[address] = cellStyles[address] || {};
    Object.assign(cellStyles[address], style);
  };
  const { choosing, selectingZone, copyingZone, cutting, matchingCells, matchingCellIndex, table, autofillDraggingTo } = store;
  {
    const { top, left, bottom, right } = zoneToArea(selectingZone);
    for (let y = top; y <= bottom; y++) {
      updateStyle({ y, x: left - 1 }, { borderRight: BORDER_SELECTED });
      updateStyle({ y, x: left }, { borderLeft: BORDER_SELECTED });
      updateStyle({ y, x: right }, { borderRight: BORDER_SELECTED });
      updateStyle({ y, x: right + 1 }, { borderLeft: BORDER_SELECTED });
    }
    for (let x = left; x <= right; x++) {
      updateStyle({ y: top - 1, x }, { borderBottom: BORDER_SELECTED });
      updateStyle({ y: top, x }, { borderTop: BORDER_SELECTED });
      updateStyle({ y: bottom, x }, { borderBottom: BORDER_SELECTED });
      updateStyle({ y: bottom + 1, x }, { borderTop: BORDER_SELECTED });
    }
  }
  if (autofillDraggingTo) {
    const autofill = new Autofill(store, autofillDraggingTo);
    const { top, left, bottom, right } = autofill.wholeArea;
    for (let y = top; y <= bottom; y++) {
      updateStyle({ y, x: left - 1 }, { borderRight: AUTOFILL_BORDER });
      updateStyle({ y, x: left }, { borderLeft: AUTOFILL_BORDER });
      updateStyle({ y, x: right }, { borderRight: AUTOFILL_BORDER });
      updateStyle({ y, x: right + 1 }, { borderLeft: AUTOFILL_BORDER });
    }
    for (let x = left; x <= right; x++) {
      updateStyle({ y: top - 1, x }, { borderBottom: AUTOFILL_BORDER });
      updateStyle({ y: top, x }, { borderTop: AUTOFILL_BORDER });
      updateStyle({ y: bottom, x }, { borderBottom: AUTOFILL_BORDER });
      updateStyle({ y: bottom + 1, x }, { borderTop: AUTOFILL_BORDER });
    }
  }
  {
    const { y, x } = choosing;
    updateStyle(
      { y, x },
      {
        borderLeft: BORDER_POINTED,
        borderRight: BORDER_POINTED,
        borderTop: BORDER_POINTED,
        borderBottom: BORDER_POINTED
      }
    );
    updateStyle({ y, x: x - 1 }, { borderRight: BORDER_POINTED });
    updateStyle({ y, x: x + 1 }, { borderLeft: BORDER_POINTED });
    updateStyle({ y: y - 1, x }, { borderBottom: BORDER_POINTED });
    updateStyle({ y: y + 1, x }, { borderTop: BORDER_POINTED });
  }
  {
    const borderStyle = cutting ? BORDER_CUTTING : BORDER_COPYING;
    const { top, left, bottom, right } = zoneToArea(copyingZone);
    for (let y = top; y <= bottom; y++) {
      updateStyle({ y, x: left - 1 }, { borderRight: borderStyle });
      updateStyle({ y, x: left }, { borderLeft: borderStyle });
      updateStyle({ y, x: right }, { borderRight: borderStyle });
      updateStyle({ y, x: right + 1 }, { borderLeft: borderStyle });
    }
    for (let x = left; x <= right; x++) {
      updateStyle({ y: top - 1, x }, { borderBottom: borderStyle });
      updateStyle({ y: top, x }, { borderTop: borderStyle });
      updateStyle({ y: bottom, x }, { borderBottom: borderStyle });
      updateStyle({ y: bottom + 1, x }, { borderTop: borderStyle });
    }
  }
  Object.entries(refs).forEach(([ref, i]) => {
    const palette = REF_PALETTE[i % REF_PALETTE.length];
    const borderStyle = `dashed 2px ${palette}`;
    const { top, left, bottom, right } = table.rangeToArea(ref);
    for (let y = top; y <= bottom; y++) {
      updateStyle({ y, x: left - 1 }, { borderRight: borderStyle });
      updateStyle({ y, x: left }, { borderLeft: borderStyle });
      updateStyle({ y, x: right }, { borderRight: borderStyle });
      updateStyle({ y, x: right + 1 }, { borderLeft: borderStyle });
    }
    for (let x = left; x <= right; x++) {
      updateStyle({ y: top - 1, x }, { borderBottom: borderStyle });
      updateStyle({ y: top, x }, { borderTop: borderStyle });
      updateStyle({ y: bottom, x }, { borderBottom: borderStyle });
      updateStyle({ y: bottom + 1, x }, { borderTop: borderStyle });
    }
  });
  matchingCells.forEach((address) => {
    const { y, x } = a2p(address);
    updateStyle({ y, x }, { backgroundColor: SEARCH_MATCHING_BACKGROUND });
  });
  if (matchingCells.length > 0) {
    const { y, x } = a2p(matchingCells[matchingCellIndex]);
    updateStyle(
      { y, x },
      {
        borderLeft: SEARCH_MATCHING_BORDER,
        borderRight: SEARCH_MATCHING_BORDER,
        borderTop: SEARCH_MATCHING_BORDER,
        borderBottom: SEARCH_MATCHING_BORDER
      }
    );
    updateStyle({ y, x: x - 1 }, { borderRight: SEARCH_MATCHING_BORDER });
    updateStyle({ y, x: x + 1 }, { borderLeft: SEARCH_MATCHING_BORDER });
    updateStyle({ y: y - 1, x }, { borderBottom: SEARCH_MATCHING_BORDER });
    updateStyle({ y: y + 1, x }, { borderTop: SEARCH_MATCHING_BORDER });
  }
  return cellStyles;
};
export {
  Tabular,
  createTableRef
};
//# sourceMappingURL=Tabular.js.map
