import { Table } from '../lib/table';
import { PointType } from '../types';
type EvaluateProps = {
    table: Table;
};
export declare class FormulaError {
    code: string;
    message: string;
    error?: Error;
    constructor(code: string, message: string, error?: Error);
}
declare class Entity<T = any> {
    value: T;
    constructor(value: T);
}
export declare class ValueEntity extends Entity {
    evaluate(): any;
}
export declare class UnreferencedEntity extends Entity {
    evaluate(): void;
}
export declare class InvalidRefEntity extends Entity<string> {
    evaluate(): void;
}
export declare class RefEntity extends Entity<string> {
    constructor(value: string);
    stringify(): string;
    private parse;
    evaluate({ table }: EvaluateProps): Table;
    id(table: Table): string;
}
export declare class RangeEntity extends Entity<string> {
    private parse;
    stringify(): string;
    evaluate({ table }: EvaluateProps): Table;
    idRange(table: Table): string;
}
export declare class IdEntity extends Entity<string> {
    private parse;
    evaluate({ table }: EvaluateProps): Table;
    ref(table: Table, slideY?: number, slideX?: number): string | undefined;
    slide(table: Table, slideY?: number, slideX?: number): string | undefined;
}
export declare class IdRangeEntity extends Entity<string> {
    private parse;
    evaluate({ table }: EvaluateProps): Table;
    range(table: Table, slideY?: number, slideX?: number): string;
    slide(table: Table, slideY?: number, slideX?: number): string;
}
export declare class FunctionEntity {
    args: Expression[];
    name: string;
    precedence: number;
    private origin?;
    constructor(name: string, precedence?: number, args?: Expression[], origin?: PointType);
    evaluate({ table }: EvaluateProps): any;
}
export type Expression = ValueEntity | RefEntity | RangeEntity | IdEntity | IdRangeEntity | FunctionEntity | UnreferencedEntity | InvalidRefEntity;
export type TokenType = 'VALUE' | 'REF' | 'RANGE' | 'ID' | 'ID_RANGE' | 'FUNCTION' | 'PREFIX_OPERATOR' | 'INFIX_OPERATOR' | 'POSTFIX_OPERATOR' | 'OPEN' | 'CLOSE' | 'COMMA' | 'SPACE' | 'UNREFERENCED' | 'INVALID_REF';
export declare class Token {
    type: TokenType;
    entity: any;
    precedence: number;
    private origin?;
    constructor(type: TokenType, entity: any, precedence?: number, origin?: PointType);
    length(): number;
    stringify(): string;
    convert(): ValueEntity | UnreferencedEntity | InvalidRefEntity | RefEntity | RangeEntity | IdEntity | IdRangeEntity | FunctionEntity | undefined;
}
type LexerOption = {
    origin?: PointType;
};
export declare class Lexer {
    private index;
    private formula;
    tokens: Token[];
    foreign: boolean;
    private origin?;
    constructor(formula: string, options?: LexerOption);
    private isWhiteSpace;
    private next;
    private get;
    private getToken;
    getTokenIndexByCharPosition(pos: number): number;
    getTokenPositionRange(index: number, slide?: number): [number, number];
    stringify(): string;
    stringifyToId(table: Table, slideY?: number, slideX?: number): string;
    stringifyToRef(table: Table): string;
    tokenize(): void;
    private skipSpaces;
    private getString;
}
export declare class Parser {
    index: number;
    depth: number;
    tokens: Token[];
    constructor(tokens: Token[]);
    build(): Expression | undefined;
    private parse;
}
export declare const absolutizeFormula: ({ value, table, slideY, slideX, }: {
    value: any;
    table: Table;
    slideY?: number;
    slideX?: number;
}) => any;
export declare const stripSheetName: (sheetName: string) => string;
export {};
//# sourceMappingURL=evaluator.d.ts.map