import dayjs from "../../_virtual/dayjs.min.js";
import { FormulaError } from "../evaluator.js";
import { BaseFunction } from "./__base.js";
import { stripTable, ensureNumber } from "./__utils.js";
import { Table } from "../../lib/table.js";
import { TimeDelta } from "../../lib/time.js";
import { SECONDS_IN_DAY } from "../../constants.js";
class AddFunction extends BaseFunction {
  constructor() {
    super(...arguments);
    this.example = "ADD(2, 3)";
    this.helpText = ["Returns the sum of two numbers.", "This is the same as the '+' operator."];
    this.helpArgs = [
      { name: "value1", description: "First additive." },
      { name: "value2", description: "Second additive." }
    ];
  }
  validate() {
    if (this.bareArgs.length !== 2) {
      throw new FormulaError("#N/A", "Number of arguments for ADD is incorrect.");
    }
    this.bareArgs = this.bareArgs.map((arg) => {
      if (arg instanceof Table) {
        arg = stripTable(arg, 0, 0);
      }
      return typeof arg === "object" ? arg : ensureNumber(arg);
    });
  }
  main(v1, v2) {
    if (typeof v1 === "number" && typeof v2 === "number") {
      return v1 + v2;
    }
    if (v1 instanceof Date && TimeDelta.is(v2)) {
      return TimeDelta.ensure(v2).add(v1);
    }
    if (TimeDelta.is(v1) && v2 instanceof Date) {
      return TimeDelta.ensure(v1).add(v2);
    }
    if (v1 instanceof Date && typeof v2 === "number") {
      return dayjs(v1).add(v2 * SECONDS_IN_DAY, "second").toDate();
    }
    if (typeof v1 === "number" && v2 instanceof Date) {
      return dayjs(v2).add(v1 * SECONDS_IN_DAY, "second").toDate();
    }
    if (!v1) {
      return v2;
    }
    if (!v2) {
      return v1;
    }
    throw new FormulaError("#VALUE!", "Mismatched types for augend and addend.");
  }
}
export {
  AddFunction
};
//# sourceMappingURL=add.js.map
