import { FormulaError } from "../evaluator.js";
import { BaseFunction } from "./__base.js";
import { ensureNumber } from "./__utils.js";
class AsinFunction extends BaseFunction {
  constructor() {
    super(...arguments);
    this.example = "ASIN(0)";
    this.helpText = ["Returns the inverse sin of the value in radians."];
    this.helpArgs = [
      {
        name: "value",
        description: "A value for the inverse sin between -1 and 1."
      }
    ];
  }
  validate() {
    if (this.bareArgs.length !== 1) {
      throw new FormulaError("#N/A", "Number of arguments for ASIN is incorrect.");
    }
    this.bareArgs = this.bareArgs.map((arg) => ensureNumber(arg));
    if (-1 > this.bareArgs[0] || this.bareArgs[0] > 1) {
      throw new FormulaError("#NUM!", "value must be between -1 and 1");
    }
  }
  main(value) {
    return Math.asin(value);
  }
}
export {
  AsinFunction
};
//# sourceMappingURL=asin.js.map
