import { FormulaError } from "../evaluator.js";
import { solveTable } from "../solver.js";
import { BaseFunction } from "./__base.js";
import { ensureString, check } from "./__utils.js";
class CountifFunction extends BaseFunction {
  constructor() {
    super(...arguments);
    this.example = 'COUNTIF(A1:A10,">20")';
    this.helpText = ["Returns the count of a series of cells."];
    this.helpArgs = [
      { name: "range", description: "Target range." },
      {
        name: "condition",
        description: "A condition for count."
      }
    ];
  }
  validate() {
    if (this.bareArgs.length !== 2) {
      throw new FormulaError("#N/A", "Number of arguments for COUNTIF is incorrect.");
    }
    this.bareArgs[1] = ensureString(this.bareArgs[1]);
  }
  main(table, condition) {
    const matrix = solveTable({ table });
    return matrix.reduce((a, b) => a.concat(b)).filter((v) => check(v, condition)).length;
  }
}
export {
  CountifFunction
};
//# sourceMappingURL=countif.js.map
