import { FormulaError } from "../evaluator.js";
import { BaseFunction } from "./__base.js";
import { ensureNumber } from "./__utils.js";
class ExpFunction extends BaseFunction {
  constructor() {
    super(...arguments);
    this.example = "EXP(2)";
    this.helpText = ["Returns the power of a number whose base is the Euler number e."];
    this.helpArgs = [
      {
        name: "exponent",
        description: "It is an exponent of power with e as the base."
      }
    ];
  }
  validate() {
    if (this.bareArgs.length !== 1) {
      throw new FormulaError("#N/A", "Number of arguments for EXP is incorrect.");
    }
    this.bareArgs = this.bareArgs.map((arg) => ensureNumber(arg));
  }
  main(exponent) {
    return Math.exp(exponent);
  }
}
export {
  ExpFunction
};
//# sourceMappingURL=exp.js.map
