import { FormulaError } from "../evaluator.js";
import { BaseFunction } from "./__base.js";
import { ensureString } from "./__utils.js";
class LenbFunction extends BaseFunction {
  constructor() {
    super(...arguments);
    this.example = "LENB(A2)";
    this.helpText = ["Returns the number of bytes in the length of the string."];
    this.helpArgs = [
      {
        name: "text",
        description: "A text to be returned the length of the bytes."
      }
    ];
  }
  validate() {
    if (this.bareArgs.length !== 1) {
      throw new FormulaError("#N/A", "Number of arguments for LENB is incorrect.");
    }
    this.bareArgs = [ensureString(this.bareArgs[0])];
  }
  main(text) {
    return encodeURIComponent(text).replace(/%../g, "x").length;
  }
}
export {
  LenbFunction
};
//# sourceMappingURL=lenb.js.map
