import { FormulaError } from "../evaluator.js";
import { solveTable } from "../solver.js";
import { Table } from "../../lib/table.js";
import { BaseFunction } from "./__base.js";
import { ensureNumber } from "./__utils.js";
class MaxFunction extends BaseFunction {
  constructor() {
    super(...arguments);
    this.example = "MAX(A2:A100, 101)";
    this.helpText = ["Returns the max in a series of numbers or cells."];
    this.helpArgs = [
      { name: "value1", description: "First number or range." },
      {
        name: "value2",
        description: "Additional numbers or ranges",
        optional: true,
        iterable: true
      }
    ];
  }
  validate() {
    if (this.bareArgs.length === 0) {
      throw new FormulaError("#N/A", "Number of arguments must be greater than 0.");
    }
    const spreaded = [];
    this.bareArgs.map((arg) => {
      if (arg instanceof Table) {
        spreaded.push(
          ...solveTable({ table: arg }).reduce((a, b) => a.concat(b)).filter((v) => typeof v === "number")
        );
        return;
      }
      spreaded.push(ensureNumber(arg));
    });
    this.bareArgs = spreaded;
  }
  main(...values) {
    if (values.length === 0) {
      return 0;
    }
    return Math.max(...values);
  }
}
export {
  MaxFunction
};
//# sourceMappingURL=max.js.map
