import { FormulaError } from "../evaluator.js";
import { BaseFunction } from "./__base.js";
import { ensureNumber } from "./__utils.js";
class ModFunction extends BaseFunction {
  constructor() {
    super(...arguments);
    this.example = "MOD(10, 4)";
    this.helpText = ["Returns the result of the modulo operation."];
    this.helpArgs = [
      {
        name: "dividend",
        description: "A number that will be divided by divisor."
      },
      { name: "divisor", description: "A number that will divide a dividend." }
    ];
  }
  validate() {
    if (this.bareArgs.length !== 2) {
      throw new FormulaError("#N/A", "Number of arguments for MOD is incorrect.");
    }
    this.bareArgs = this.bareArgs.map((arg) => ensureNumber(arg));
    if (this.bareArgs[1] === 0) {
      throw new FormulaError("#DIV/0!", "The second argument must be non-zero.");
    }
  }
  main(v1, v2) {
    return (v1 % v2 + v2) % v2;
  }
}
export {
  ModFunction
};
//# sourceMappingURL=mod.js.map
