import { FormulaError } from "../evaluator.js";
import { BaseFunction } from "./__base.js";
import { ensureNumber } from "./__utils.js";
class MultiplyFunction extends BaseFunction {
  constructor() {
    super(...arguments);
    this.example = "MULTIPLY(6, 7)";
    this.helpText = ["Returns the product of two numbers.", "This is the same as the '*' operator."];
    this.helpArgs = [
      { name: "factor1", description: "First factor." },
      { name: "factor2", description: "Second factor." }
    ];
  }
  validate() {
    if (this.bareArgs.length !== 2) {
      throw new FormulaError("#N/A", "Number of arguments for MULTIPLY is incorrect.");
    }
    this.bareArgs = this.bareArgs.map((arg) => ensureNumber(arg));
  }
  main(v1, v2) {
    return v1 * v2;
  }
}
export {
  MultiplyFunction
};
//# sourceMappingURL=multiply.js.map
