import { FormulaError } from "../evaluator.js";
import { BaseFunction } from "./__base.js";
import { ensureNumber } from "./__utils.js";
class RoundFunction extends BaseFunction {
  constructor() {
    super(...arguments);
    this.example = "ROUND(99.44,1)";
    this.helpText = ["Round a number to the specified number of decimal places according to standard rules."];
    this.helpArgs = [
      {
        name: "value",
        description: "A number to be rounded."
      },
      {
        name: "digit",
        description: "The number of decimal places after rounding.",
        optional: true
      }
    ];
  }
  validate() {
    if (this.bareArgs.length !== 1 && this.bareArgs.length !== 2) {
      throw new FormulaError("#N/A", "Number of arguments for ROUND is incorrect.");
    }
    this.bareArgs = this.bareArgs.map((arg) => ensureNumber(arg));
  }
  main(value, digit = 0) {
    const multiplier = Math.pow(10, digit);
    return Math.round(value * multiplier) / multiplier;
  }
}
export {
  RoundFunction
};
//# sourceMappingURL=round.js.map
