import { FormulaError } from "../evaluator.js";
import { BaseFunction } from "./__base.js";
import { ensureNumber } from "./__utils.js";
class SinFunction extends BaseFunction {
  constructor() {
    super(...arguments);
    this.example = "SIN(PI()/2)";
    this.helpText = ["Returns the sin of the angle specified in radians."];
    this.helpArgs = [
      {
        name: "angle",
        description: "An angle in radians, at which you want the sin."
      }
    ];
  }
  validate() {
    if (this.bareArgs.length !== 1) {
      throw new FormulaError("#N/A", "Number of arguments for SIN is incorrect.");
    }
    this.bareArgs = this.bareArgs.map((arg) => ensureNumber(arg));
  }
  main(angle) {
    return Math.sin(angle);
  }
}
export {
  SinFunction
};
//# sourceMappingURL=sin.js.map
