import { FormulaError } from "../evaluator.js";
import { BaseFunction } from "./__base.js";
import { ensureNumber } from "./__utils.js";
class SqrtFunction extends BaseFunction {
  constructor() {
    super(...arguments);
    this.example = "SQRT(9)";
    this.helpText = ["Returns the positive square root of a positive number."];
    this.helpArgs = [
      {
        name: "value",
        description: "A number for which the positive square root is to be found."
      }
    ];
  }
  validate() {
    if (this.bareArgs.length !== 1) {
      throw new FormulaError("#N/A", "Number of arguments for SQRT is incorrect.");
    }
    this.bareArgs = this.bareArgs.map((arg) => ensureNumber(arg));
    if (this.bareArgs[0] < 0) {
      throw new FormulaError("NUM!", "First argument must be positive.");
    }
  }
  main(value) {
    return Math.sqrt(value);
  }
}
export {
  SqrtFunction
};
//# sourceMappingURL=sqrt.js.map
