import { Special } from "../constants.js";
import { Table } from "../lib/table.js";
import { Lexer, Parser, FormulaError } from "./evaluator.js";
import { p2a } from "../lib/converters.js";
const SOLVING = new Special("solving");
const solveFormula = ({ value, table, raise = true, evaluates = true, origin }) => {
  var _a;
  if (evaluates === null) {
    return value;
  }
  let solved = value;
  if (typeof value === "string") {
    if (value.charAt(0) === "=") {
      try {
        const lexer = new Lexer(value.substring(1), { origin });
        lexer.tokenize();
        const parser = new Parser(lexer.tokens);
        if (evaluates === false) {
          return "=" + lexer.stringifyToRef(table);
        }
        const expr = parser.build();
        solved = (_a = expr == null ? void 0 : expr.evaluate) == null ? void 0 : _a.call(expr, { table });
      } catch (e) {
        if (raise) {
          throw e;
        }
        return null;
      }
    }
  }
  if (solved instanceof Table) {
    solved = solveTable({ table: solved, raise })[0][0];
  }
  return solved;
};
const solveTable = ({ table, raise = true }) => {
  const area = table.getArea();
  return table.getMatrixFlatten({ area, evaluates: null }).map((row, i) => {
    const y = area.top + i;
    return row.map((value, j) => {
      const x = area.left + j;
      const address = p2a({ y, x });
      const cache = table.getSolvedCache(address);
      try {
        if (cache === SOLVING) {
          throw new FormulaError("#REF!", "References are circulating.", new Error(value));
        } else if (cache instanceof FormulaError) {
          throw cache;
        } else if (cache != null) {
          return cache;
        }
        table.setSolvedCache(address, SOLVING);
        const solved = solveFormula({ value, table, raise, origin: { y, x } });
        table.setSolvedCache(address, solved);
        return solved;
      } catch (e) {
        table.setSolvedCache(address, e);
        if (raise) {
          throw e;
        }
        return null;
      }
    });
  });
};
export {
  solveFormula,
  solveTable
};
//# sourceMappingURL=solver.js.map
