import dayjs from "../_virtual/dayjs.min.js";
const defaultTimeDeltaFormat = "HH:mm:ss";
const BASE_DATE = /* @__PURE__ */ new Date("2345-01-02T03:04:05Z");
const UNITS = ["year", "month", "day", "hour", "minute", "second", "millisecond"];
const ADD_FNS = UNITS.map(
  (unit) => (date, amount) => dayjs(date).add(amount, unit).toDate()
);
const SUB_FNS = UNITS.map(
  (unit) => (date, amount) => dayjs(date).subtract(amount, unit).toDate()
);
class TimeDelta {
  constructor(date1, date2, format = defaultTimeDeltaFormat) {
    this.gsType = "TimeDelta";
    this.diff = [0, 0, 0, 0, 0, 0, 0];
    this.diff = [
      date1.getFullYear() - date2.getFullYear(),
      date1.getMonth() - date2.getMonth(),
      date1.getDate() - date2.getDate(),
      date1.getHours() - date2.getHours(),
      date1.getMinutes() - date2.getMinutes(),
      date1.getSeconds() - date2.getSeconds(),
      date1.getMilliseconds() - date2.getMilliseconds()
    ];
    this.date1 = date1;
    this.date2 = date2;
    this.format = format;
  }
  add(date) {
    this.diff.forEach((n, i) => {
      date = ADD_FNS[i](date, n);
    });
    return date;
  }
  sub(date) {
    this.diff.forEach((n, i) => {
      date = SUB_FNS[i](date, n);
    });
    return date;
  }
  stringify(format) {
    if (format == null) {
      format = this.format;
    }
    const tokens = [];
    const msecs = this.date1.getMilliseconds() - this.date2.getMilliseconds();
    let secs = (this.date1.getTime() - this.date2.getTime()) / 1e3;
    for (const divider of [3600, 60]) {
      tokens.push(Math.floor(secs / divider));
      secs %= divider;
    }
    tokens.push(secs, msecs);
    let result = format;
    result = result.replace("HH", String(tokens[0]).padStart(2, "0"));
    result = result.replace("H", String(tokens[0]));
    result = result.replace("mm", String(tokens[1]).padStart(2, "0"));
    result = result.replace("ss", String(tokens[2]).padStart(2, "0"));
    result = result.replace("SSS", String(tokens[3]).padStart(3, "0"));
    result = result.replace("SS", String(tokens[3]).padStart(2, "0").substring(0, 2));
    result = result.replace("S", String(tokens[3]).padStart(1, "0").substring(0, 1));
    return result;
  }
  toJSON() {
    return this.stringify();
  }
  toString() {
    return this.stringify();
  }
  static create(hours = 0, minutes = 0, seconds = 0, milliseconds = 0) {
    const diff = [0, 0, 0, hours, minutes, seconds, milliseconds];
    let date = BASE_DATE;
    diff.forEach((n, i) => {
      date = ADD_FNS[i](date, n);
    });
    return new TimeDelta(date, BASE_DATE);
  }
  static is(obj) {
    if (obj instanceof TimeDelta) {
      return true;
    }
    if ((obj == null ? void 0 : obj.gsType) === "TimeDelta") {
      return true;
    }
    return false;
  }
  static ensure(obj) {
    if (obj instanceof TimeDelta) {
      return obj;
    }
    if ((obj == null ? void 0 : obj.gsType) === "TimeDelta") {
      return TimeDelta.fromObject(obj);
    }
    return TimeDelta.create();
  }
  static fromObject(obj) {
    return new TimeDelta(new Date(obj.date1), new Date(obj.date2));
  }
  static parse(value, format = defaultTimeDeltaFormat, strict = false) {
    {
      const formattedMatcher = dayjsFormatToNamedRegex(format);
      const match = value.match(formattedMatcher);
      if (match == null ? void 0 : match.groups) {
        return TimeDelta.create(
          Number(match.groups.HH || match.groups.H || 0),
          Number(match.groups.mm || match.groups.m || 0),
          Number(match.groups.ss || match.groups.s || 0),
          Number(match.groups.SSS || match.groups.SS || match.groups.S || 0)
        );
      }
    }
    if (strict) {
      return;
    }
    {
      const match = value.match(/^([+-]?)(\d+):(\d{2})$/);
      if (match) {
        const [, _sign, hours, minutes] = match;
        const sign = _sign === "-" ? -1 : 1;
        return TimeDelta.create(sign * Number(hours), sign * Number(minutes));
      }
    }
    {
      const match = value.match(/^([+-]?)(\d+):(\d{2}):(\d{2})$/);
      if (match) {
        const [, _sign, hours, minutes, seconds] = match;
        const sign = _sign === "-" ? -1 : 1;
        return TimeDelta.create(sign * Number(hours), sign * Number(minutes), sign * Number(seconds));
      }
    }
    {
      const match = value.match(/^([+-]?)(\d+):(\d{2}):(\d{2})\.(\d+)$/);
      if (match) {
        const [, _sign, hours, minutes, seconds, msecs] = match;
        const sign = _sign === "-" ? -1 : 1;
        return TimeDelta.create(
          sign * Number(hours),
          sign * Number(minutes),
          sign * Number(seconds),
          sign * Number(msecs)
        );
      }
    }
  }
}
const tokenRegexMap = {
  HH: { group: "HH", pattern: "(?<HH>\\d+)" },
  H: { group: "H", pattern: "(?<HH>\\d+)" },
  mm: { group: "mm", pattern: "(?<mm>[0-5]\\d)" },
  m: { group: "m", pattern: "(?<m>\\d|[1-5]\\d)" },
  ss: { group: "ss", pattern: "(?<ss>[0-5]\\d)" },
  s: { group: "s", pattern: "(?<s>\\d|[1-5]\\d)" },
  SSS: { group: "SSS", pattern: "(?<SSS>\\d{3})" },
  SS: { group: "SS", pattern: "(?<SS>\\d{2})" },
  S: { group: "S", pattern: "(?<S>\\d)" }
};
function dayjsFormatToNamedRegex(format) {
  const sortedTokens = Object.keys(tokenRegexMap).sort((a, b) => b.length - a.length);
  const tokenPattern = new RegExp(sortedTokens.join("|"), "g");
  const escapedFormat = format.replace(/[-\/\\^$*+?.()|[\]{}]/g, "\\$&");
  const regexSource = escapedFormat.replace(tokenPattern, (match) => {
    var _a;
    return ((_a = tokenRegexMap[match]) == null ? void 0 : _a.pattern) ?? match;
  });
  return new RegExp(`^${regexSource}$`);
}
export {
  BASE_DATE,
  TimeDelta,
  defaultTimeDeltaFormat
};
//# sourceMappingURL=time.js.map
